// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga;

import static org.makagiga.common.UI._;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.Date;
import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.tree.DefaultTreeCellRenderer;

import org.makagiga.common.*;
import org.makagiga.editors.EditorPlugin;
import org.makagiga.plugins.PluginInfo;

/**
 * Meta Info.
 * Each tree item or tab has associated its own meta info.
 * Meta info contains additional information, such as file color or comment.
 */
public final class MetaInfo {

	// public

	/**
	 * Enabled/disables bookmark.
	 * If @c true, the bookmark is visible in the "Bookmarks" menu.
	 */
	public final BooleanProperty bookmark = new BooleanProperty();

	/**
	 * @c true if this folder is expanded.
	 */
	public final BooleanProperty expanded = new BooleanProperty();

	/**
	 * @c true if file is currently being edited,
	 * and should be automatically opened on next application startup.
	 */
	public final BooleanProperty open = new BooleanProperty();

	// attributes
	public static final int DATA_ROOT = 1 << 0; /**< Data root folder. */
	public static final int FOLDER = 1 << 1; /**< Folder. */
	public static final int FILE = 1 << 2; /**< Data file. */
	public static final int NOT_SUPPORTED = 1 << 3; /**< File type is not supported. */
	public static final int MAIN_ROOT = 1 << 4; /**< Invisible main root folder for data and trash roots. */
	public static final int TRASH_ROOT = 1 << 5; /**< Trash root folder. */

	// private

	private boolean locked;
	private static boolean toolTipsEnabled;
	private Color color;
	private static Color listSelectionBackground = UI.getUIColor("List.selectionBackground");
	private static Color listSelectionForeground = UI.getUIColor("List.selectionForeground");
	private Config config;
	private Icon icon;
	private static Image lockedMiniIcon;
	private ImageIcon lockedIconCache;
	private int attr;
	private int usage;
	private long createdTime;
	private long openTime;
	private PluginInfo info;
	private String comment;
	private String htmlToolTipCache;
	private String iconName;
	private String name;
	private String path;

	// public

	/**
	 * Constructs a new meta info.
	 * @param path A full file name
	 * @param attr Attributes (e.g. @c FOLDER)
	 */
	public MetaInfo(final String path, final int attr) {
		this.attr = attr;
		setPath(path);
		if (isFile())
			loadFromFile(this.path + ".properties");
		else if (isFolder())
			loadFromFile(FS.makePath(this.path, ".properties"));
		else if (isDataRoot())
			loadFromFile(null);
	}

	/**
	 * Returns the file age in days.
	 */
	public long getAge() {
		long lastModified = (createdTime == 0) ? new File(path).lastModified() : createdTime;
		long age = DT.now().getTime() - lastModified;
		long days = (age / 1000) / 3600 / 24;

		return (days == 0) ? 1 : days;
	}

	/**
	 * Returns color.
	 */
	public Color getColor() { return color; }

	/**
	 * Sets color to @p value.
	 */
	public void setColor(final Color value) {
		color = value;
		htmlToolTipCache = null;
	}

	/**
	 * Returns comment.
	 */
	public String getComment() { return comment; }

	/**
	 * Sets comment to @p value.
	 */
	public void setComment(final String value) {
		comment = value.trim();
		htmlToolTipCache = null;
	}

	/**
	 * Returns a configuration file (*.properties)
	 * associated with this file.
	 */
	public Config getConfig() { return config; }

	/**
	 * Returns escaped name.
	 */
	public String getEscapedName() { return name; }

	/**
	 * Returns icon associated with this file.
	 */
	public Icon getIcon() {//!!!sometimes "lock" icon overlay is not painted, Check image size in "locked" icon [0.9.3]
		// no icon?
		if (icon == null) {
			// no plugin
			if (iconName == null)
				return null;

			// load custom icon
			icon = FS.getIcon(iconName);
			// no custom icon, load safe icon
			if (icon == null) {
				iconName = "misc";
				icon = FS.getIcon(iconName);
			}
		}
		if (locked) {
			if (lockedIconCache != null)
				return lockedIconCache;

			int w = icon.getIconWidth() / 2;
			int h = icon.getIconHeight() / 2;
			if (lockedMiniIcon == null) {
				lockedMiniIcon = FS.getImage("locked");
				lockedMiniIcon = lockedMiniIcon.getScaledInstance(w, h, Image.SCALE_SMOOTH);
			}
			BufferedImage bi = new BufferedImage(
				icon.getIconWidth(),
				icon.getIconHeight(),
				BufferedImage.TYPE_INT_ARGB
			);
			// draw "locked" overlay icon
			Graphics g = bi.createGraphics();
			ImageIcon imageIcon = ImageIcon.class.cast(icon);
			g.drawImage(imageIcon.getImage(), 0, 0, null, null);
			g.drawImage(lockedMiniIcon, w, h, null, null);
			lockedIconCache = new ImageIcon(bi);
			lockedIconCache.setDescription(imageIcon.toString());
			g.dispose();

			return lockedIconCache;
		}

		return icon;
	}

	/**
	 * Sets icon to @p value.
	 */
	public void setIcon(final Icon value) {
		if (value == null) {
			iconName = isFolder() ? "ui/folder" : "ui/file";
			icon = FS.getIcon(iconName);
		}
		else {
			iconName = null;
			icon = value;
		}
		htmlToolTipCache = null;
		lockedIconCache = null;
	}

	public String getKeywords() {
		return config.read("keywords");
	}

	public void setKeywords(final String value) {
		// MLogger.info("%s: %s", path, value);
		config.write("keywords", value);
	}

	/**
	 * Returns a "nice" representation of this item.
	 * This function should be used only in @b GUI-display related functions.
	 */
	public String getNicePath() {
// TODO: 2.0: common code for MAIN_ROOT and "vfs/data", abstract VFS for trash, data, etc.
		String relativePath =
			isDataRoot() || isMainRoot()
			? path
			: path.substring(FS.getConfigDir().length() + "vfs/data".length() + 2);
		String nicePath = getPathWithoutExtension(relativePath);

		return TK.unescapeURL(nicePath.replace(File.separator, " / "));
	}

	/**
	 * Returns path (full file name).
	 */
	public String getPath() { return path; }

	/**
	 * Sets path to @p value.
	 */
	public void setPath(final String value) {
		path = value;
		if (isMainRoot()) {
			// MAIN_ROOT has no path
			name = ""; // must be non-null
		}
		else {
			File file = new File(path);
			name = isFile() ? getPathWithoutExtension(file.getName()) : file.getName();
			htmlToolTipCache = null;
		}
	}

	public float getRatio() {
		return isFolder() ? -1.0f : (getUsage() / getAge());
	}

	/**
	 * Returns tool tip text.
	 * If @c htmlToolTipCache is @c null, the tool tip is regenerated;
	 * otherwise the @c htmlToolTipCache is returned.
	 */
	public String getToolTipText() {
		if (!toolTipsEnabled)
			return null;

		if (htmlToolTipCache != null)
			return htmlToolTipCache;

		String colorTags;
		if (color == null) {
			colorTags = "";
		}
		else {
			colorTags = "&nbsp;&nbsp;<span style=\"background-color:" + TK.toHTMLTriplet(color) + ";\">&nbsp;&nbsp;&nbsp;&nbsp;</span>";
		}
		String location = getNicePath();
		location = location.replace(File.separator, "<span style=\"color: gray;\">/</span>");
		// \n - debug purpose
		StringBuilder html = new StringBuilder(1024);
		html.append("<table border=\"0\">\n");
		html.append("<tr><td>" + colorTags);
		html.append("&nbsp;&nbsp;<font size=\"+1\">" + this + "</font></td></tr>\n");
		// type
		String type = getType();
		if (type == null) { // folder
			type = _("Folder");
		}
		else { // file
			if (info == null)
				info = EditorPlugin.findPluginForType(type);
			type = (info == null) ? _("Unknown") : info.name.get();
		}
		html.append(String.format(
			"<tr><td><b>%s</b> %s</td></tr>\n",
			_("Type:"),
			type
		));
		// comment
		if (!TK.isEmpty(comment)) {
			// get first line from a multiline comment
			int n = comment.indexOf('\n');
			String s = (n == -1) ? comment : comment.substring(0, n) + "...";
			// limit comment length to MAX_COMMENT_LEN
			final int MAX_COMMENT_LEN = 128;
			if (s.length() > MAX_COMMENT_LEN)
				s = s.substring(0, MAX_COMMENT_LEN) + "...";
			html.append(String.format(
				"<tr><td><b>%s</b> %s</td></tr>\n",
				_("Comment:"),
				s.replace("\n", "<br>")
			));
		}
		html.append(String.format(
			"<tr><td><b>%s</b> %s</td></tr>\n",
			_("Created:"),
			getFileCreatedDate()
		));
		html.append(String.format(
			"<tr><td><b>%s</b> %s</td></tr>\n",
			_("Modified:"),
			getFileModifiedDate()
		));
		html.append(String.format(
			"<tr><td><b>%s</b> %s</td></tr>\n",
			_("Location:"),
			location
		));
		if (locked) {
			html.append("<tr><td><font color=\"#ff0000\">" + _("This file is locked") + "</font></td></tr>\n");
		}

		if (UI.toolTipsDetails.get()) {
			// separator
			html.append("<tr><td><hr></td></tr>\n");
			if (isFile()) {
				// last used
				if (openTime != 0) {
					html.append(String.format(
						"<tr><td><b>%s</b> %s</td></tr>\n",
						_("Last Opened:"),
						formatDate(openTime)
					));
				}
				// usage
				html.append(String.format(
					"<tr><td><b>%s</b> %s</td></tr>\n",
					_("Usage:"),
					(usage == 0) ? _("Never used before") : Integer.toString(usage)
				));
			}
			// age
			html.append(String.format(
				"<tr><td><b>%s</b> %s</td></tr>\n",
				_("Age:"),
				_("{0} day(s)", getAge())
			));
		}

		html.append("</table>");

		htmlToolTipCache = UI.makeHTML(html.toString());

		return htmlToolTipCache;
	}

	/**
	 * Returns the type of this file. (e.g. ".png")
	 */
	public String getType() {
		if (isFile())
			return FS.getType(path);

		return null;
	}

	public int getUsage() { return usage; }

	public boolean isCustomIcon() { return iconName != null; }

	/**
	 * Returns @c true if this file is a <b>data root</b>.
	 */
	public boolean isDataRoot() { return (attr & DATA_ROOT) != 0; }

	/**
	 * Returns @c true if this file is a <b>data file</b>.
	 */
	public boolean isFile() { return (attr & FILE) != 0; }

	/**
	 * Returns @c true if this file is a <b>folder</b>.
	 */
	public boolean isFolder() { return (attr & FOLDER) != 0; }

	/**
	 * Returns @c true if this file is locked (read only).
	 */
	public boolean isLocked() { return locked; }

	/**
	 * Sets lock state to @p value.
	 */
	public void setLocked(final boolean value) {
		locked = value;
		htmlToolTipCache = null;
		lockedIconCache = null;
	}

	public boolean isMainRoot() { return (attr & MAIN_ROOT) != 0; }

	/**
	 * Returns @c true if this type of file is supported (has installed plugin).
	 */
	public boolean isSupported() { return (attr & NOT_SUPPORTED) == 0; }

	/**
	 * Saves meta info to file.
	 * Quick info:
	 * - Directory (folder) saves its info in ".properties" file.
	 * - File saves its info in "Foo.properties" file, where "Foo" is its file name.
	 * - Root folder has no properties.
	 */
	public void saveToFile() {
		if (config == null) {
			MLogger.error("MetaInfo.saveToFile: config == null");

			return;
		}

		config.write("bookmark", bookmark);
		config.write("color", color);
		config.write("comment", comment);
		config.write("created", new Date(createdTime));
		config.write("expanded", expanded);
		if (iconName == null)
			config.removeString("icon");
		else
			config.write("icon", iconName);
		config.write("lastOpen", new Date(openTime));
		config.write("locked", locked);
		config.write("open", open);
		config.write("usage", usage);

		if (isFolder())
			config.path.set(FS.makePath(path, ".properties"));
		else
			config.path.set(path + ".properties");
		config.saveToFile();

		htmlToolTipCache = null;
	}

	/**
	 * Sets "creation" time to @b now.
	 */
	public void setCreatedTime() {
		createdTime = DT.now().getTime();
		htmlToolTipCache = null;
	}

	/**
	 * Sets "last open" time to @b now.
	 */
	public void setLastOpenTime() {
		openTime = DT.now().getTime();
		usage++;
		htmlToolTipCache = null;
	}

	/**
	 * Enables/disables tool tips text.
	 * Due performance reasons, tool tips are disables on startup.
	 */
	public static void setToolTipsEnabled(final boolean value) {
		toolTipsEnabled = value;
	}

	public void setupRenderer(final JLabel renderer, final int items, final boolean selected, final boolean expanded, final boolean hasFocus) {
		// invisible
		if (isMainRoot())
			return;

		boolean dataRoot = isDataRoot();
		String text;

// TODO: 2.0: folder content in tool tip text (Editor.getPreview()?)
		if (dataRoot) {
			if (expanded || (items <= 0))
				text = _("Files and Folders");
			else
				text = _("Files and Folders - Click to expand");
			renderer.setToolTipText(UI.makeHTML(text));
		}
		else {
			text = toString();
			renderer.setToolTipText(getToolTipText());
		}
		if (isFolder() || dataRoot) {
			if (items != -1)
				text += " (" + ((items == 0) ? _("Empty") : items) + ")";
		}
		// colors
		if (renderer instanceof DefaultListCellRenderer) {
			if (!selected && !hasFocus)
				renderer.setBackground((color == null) ? null : color);
			else
				renderer.setBackground(listSelectionBackground);
		}
		else if (renderer instanceof DefaultTreeCellRenderer) {
			DefaultTreeCellRenderer treeRenderer = (DefaultTreeCellRenderer)renderer;
			treeRenderer.setBackgroundNonSelectionColor(color);
			treeRenderer.setBorderSelectionColor(color);
		}
		if (!selected && !hasFocus)
			renderer.setForeground(UI.getXorColor(color));
		else
			renderer.setForeground(listSelectionForeground);

		// font
		final int STYLE = open.get() ? Font.BOLD : Font.PLAIN;
		renderer.setFont(renderer.getFont().deriveFont(STYLE));
		// text
		renderer.setText(isSupported() ? text : ("[" + _("NO PLUGIN") + "] " + text));
		// icon
		renderer.setIcon(getIcon());
	}

	// private

	private String formatDate(final long date) {
		if (date == 0)
			return null;

		return DT.format(date, DT.FULL, DT.SHORT);
	}

	private String getFileCreatedDate() {
		if (createdTime == 0)
			return getFileModifiedDate();

		return formatDate(createdTime);
	}

	private String getFileModifiedDate() {
		return formatDate(new File(path).lastModified());
	}

	private String getPathWithoutExtension(final String pathWithExtension) {
		if (isFolder())
			return pathWithExtension;

		int i = pathWithExtension.lastIndexOf(".");

		if (i != -1)
			return pathWithExtension.substring(0, i);

		return pathWithExtension;
	}

	private void loadFromFile(final String path) {
		// root
		if (path == null) {
			iconName = "makagiga";

			return;
		}
		// file or folder
		config = new Config(path);
		bookmark.set(config.read("bookmark", false));
		color = config.readColor("color", null);
		comment = config.read("comment");
		createdTime = config.readDate("created", 0).getTime();
		if (createdTime == 0)
			createdTime = new File(path).lastModified();
		expanded.set(config.read("expanded", false));
		locked = config.read("locked", false); // before "icon" read
		iconName = config.read("icon", isFolder() ? "ui/folder" : null);
		openTime = config.readDate("lastOpen", 0).getTime();
		open.set(config.read("open", false));
		usage = config.readInt("usage", 0);
	}

	/**
	 * Sets icon to @p name.
	 */
	public void setIconName(final String value) {
		iconName = value;
		icon = FS.getIcon(iconName);
		if (icon == null) {
			// set default icon
			setIcon(null);

			return;
		}
		htmlToolTipCache = null;
		lockedIconCache = null;
	}

	/**
	 * Returns unescaped name.
	 * File name without directory and extension, "%20" -> " ".
	 */
	@Override
	public String toString() {
		return TK.unescapeURL(name);
	}

}
