// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga;

import static org.makagiga.common.UI._;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.makagiga.common.*;
import org.makagiga.search.Search;
import org.makagiga.tools.Browser;
import org.makagiga.tree.Tree;
import org.makagiga.tree.TreeList;

/** The Sidebar. */
public final class Sidebar extends MTabbedPane<MPanel> {

	// public

	/**
	 * The standard tabs.
	 */
	public enum Tab {
		TREE, /**< The Tree tab. */
		DESIGNER, /**< The Designer tab. */
		SEARCH, /**< The Search tab. */
		BROWSER /**< The Browser tab. */
	};

	// private

	private int lastSelectedTab;
	private static Sidebar _instance;

	// public

	/**
	 * Creates and returns an instance of the Sidebar.
	 */
	public static Sidebar getInstance() {
		if (_instance == null)
			new Sidebar(); // set "_instance" in constructor

		return _instance;
	}

	/**
	 * Switches to the @p tab.
	 */
	public static void goTo(final Tab tab) {
		final Sidebar sidebar = getInstance();
		sidebar.setVisible(true);

		if (!sidebar.isSelected(tab)) {
			if (!sidebar.isEnabledAt(tab.ordinal()) && (tab == Tab.DESIGNER))
				return;

			sidebar.setSelectedIndex(tab.ordinal());
		}

		switch (tab) {
			case TREE:
				final TreeList treeList = TreeList.getInstance();
				treeList.requestFocusInWindow();
				treeList.focusItem(Tabs.getInstance().getCurrentMetaInfo());
				break;
			case DESIGNER:
				break;
			case SEARCH:
				Search.getInstance().setFocus();
				break;
			case BROWSER:
				Browser.getInstance().getContent().requestFocusInWindow();
				break;
		}
	}

	/**
	 * Returns @c true if @b Browser is selected.
	 */
	public boolean isBrowserSelected() {
		return isSelected(Tab.BROWSER);
	}

	/**
	 * Returns @c true if @b Designer is selected.
	 */
	public boolean isDesignerSelected() {
		return isSelected(Tab.DESIGNER);
	}

	/**
	 * Returns @c true if @b Search is selected.
	 */
	public boolean isSearchSelected() {
		return isSelected(Tab.SEARCH);
	}

	/**
	 * Returns @c true if @b Tree is selected.
	 */
	public boolean isTreeSelected() {
		return isSelected(Tab.TREE);
	}

	/**
	 * Selects the last selected tab.
	 */
	public void selectLastTab() {
		setSelectedIndex(lastSelectedTab);
	}

	/**
	 * Selects a tab at @p index.
	 * This function is overriden to handle @ref selectLastTab.
	 */
	@Override
	public void setSelectedIndex(final int index) {
		if (getSelectedIndex() != index)
			lastSelectedTab = getSelectedIndex();
		super.setSelectedIndex(index);
	}

	// private

	private Sidebar() {
		_instance = this;
		setTabPlacement(BOTTOM);

		addTab(_("Tree"), "tree", Tree.getInstance());
		setToolTipTextAt(Tab.TREE.ordinal(), UI.makeHTML(_("All the files and folders.")));

		addTab(_("Designer"), "designer");
		setToolTipTextAt(Tab.DESIGNER.ordinal(), UI.makeHTML(_("Modify the content of the active editor.")));

		addTab(_("Find Files"), "find");
		setToolTipTextAt(Tab.SEARCH.ordinal(), UI.makeHTML(_("Search for files.")));

		addTab(_("Browser"), "browser");
		setEnabledAt(Tab.BROWSER.ordinal(), false);
		setToolTipTextAt(Tab.BROWSER.ordinal(), UI.makeHTML(_("Universal browser.")));

		// init some tabs on demand
		addChangeListener(new ChangeListener() {
			public void stateChanged(final ChangeEvent e) {
				initOnDemand();
			}
		} );
	}

	/**
	 * Initializes tabs on demand.
	 */
	private void initOnDemand() {
		// init browser
		if (
			(getTabAt(Tab.BROWSER.ordinal()) == null) &&
			isBrowserSelected()
		) {
			setComponentAt(Tab.BROWSER.ordinal(), Browser.getInstance());
			setEnabledAt(Tab.BROWSER.ordinal(), true);
		}
		// init search
		else if (
			(getTabAt(Tab.SEARCH.ordinal()) == null) &&
			isSearchSelected()
		) {
			final Search search = Search.getInstance();
			setComponentAt(Tab.SEARCH.ordinal(), search);
			search.setFocus();
		}
	}

	private boolean isSelected(final Tab tab) {
		return getSelectedIndex() == tab.ordinal();
	}

}
