// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.common;

import static java.lang.System.err;
import static java.lang.System.out;

import static org.makagiga.common.UI._;

import java.util.ArrayList;
import java.util.Hashtable;

/**
 * A command line arguments.
 *
 * EXAMPLE:
 * @code
 * ...
 * public static void main(String[] args) {
 * ...
 *   // Init AppInfo first!
 * ...
 *   Args.init(args);
 *   Args.add("foo", _("Do foo and exit"));
 *   Args.add("verbose", _("Enable verbose mode"));
 *   Args.check(); // check for "--help" and "--version" argument
 * ...
 * @endcode
 * @see add
 * @see check
 * @see init
 */
public final class Args {

	// private

	private static int argc;
	private static Hashtable<String, String> table = new Hashtable<String, String>();
	private static String[] argv;

	// public

	/**
	 * Adds a new item to the list of valid command line arguments.
	 * Do not add "help" and "version" arguments.
	 * It will be added automatically in @ref init(final String[]).
	 * @param name An argument name (e.g. "foo")
	 * @param description An argument description (e.g. "Do foo and exit")
	 */
	public static void add(final String name, final String description) {
		table.put(name, description);
	}

	/**
	 * Checks for the internal command line arguments such as "--help".
	 */
	public static void check() {
		if (isSet("help")) {
			help();
			TK.quit();
		}
		else if (isSet("version")) {
			version();
			TK.quit();
		}
	}

	/**
	 * Prints used command line arguments to @b stderr.
	 */
	public static void dump() {
		err.printf("Args.argc = %d\n", argc);
		for (int i = 0; i < argc; i++)
			err.printf("Args.argv[%d] = \"%s\"\n", i, argv[i]);
	}

	/**
	 * Prints the command line help to @b stdout.
	 * Arguments are sorted by its name.
	 */
	public static void help() {
		out.println();
		out.printf("%s\n", AppInfo.getTitle());
		out.printf("%s\n", AppInfo.copyright);
		out.println();
		// sort items by arg name
		ArrayList<String> sortedArray = TK.sortByKey(table, new StringComparator());
		// print sorted items
		for (String i : sortedArray)
			printArg(i, table.get(i));
		out.println();
	}

	/**
	 * Call it from the "main" function at application startup.
	 * @param args An array of the command line arguments
	 */
	public static void init(final String[] args) {
		argv = args;
		argc = argv.length;
		add("help", _("This help screen"));
		add("no-log", _("No Console log"));
		add("version", _("Show version info and exit"));
	}

	/**
	 * Returns @c true if @p arg has been passed to the command line arguments.
	 *
	 * EXAMPLE:
 	 * @code
	 * ...
	 * if (Args.isSet("foo")) // check for "--foo" argument
	 *   doFoo();
	 * ...
	 * @endcode
	 * @param arg An argument name without leading "--"
	 */
	public static boolean isSet(final String arg) {
		if (argc == 0)
			return false;

		String s = "--" + arg;
		for (String i : argv) {
			if (i.equals(s))
				return true;
		}

		return false;
	}

	/**
	 * Prints some version info to @b stdout.
	 */
	public static void version() {
		out.println();
		out.printf("%s\n", AppInfo.getTitle());
		out.printf(
			"%s %s, Java %s\n",
			OS.getName(),
			OS.getVersion(),
			System.getProperty("java.runtime.version")
		);
		out.println();
	}

	// private

	private static void printArg(final String name, final String description) {
		out.printf("--%-20s%s\n", name, description);
	}

}
