// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.common;

import static java.awt.event.KeyEvent.*;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.util.Iterator;
import javax.swing.Action;
import javax.swing.JComboBox;
import javax.swing.JTextField;

// !!!: popup menu won't hide after click on the text field
// WONTFIX: disable "arrow" button if list is empty
/** A combo box with mouse wheel support. */
public class MComboBox<I> extends JComboBox
	implements
		Iterable<I>,
		MouseWheelListener
{

	// public

	/**
	 * Constructs a new combo box.
	 */
	public MComboBox() {
		addActionListener(new ActionListener() {
			public void actionPerformed(final ActionEvent e) {
				onSelect();
			}
		} );
		addMouseWheelListener(this);
		MText.install(getTextField());
	}

	/**
	 * Removes all text.
	 */
	public void clear() {
		setText(null);
	}

	public MComboBoxIterator comboBoxIterator() {
		return (MComboBoxIterator)iterator();
	}

	@Override
	public Dimension getMaximumSize() {
		// do not stretch vertically
		return new Dimension(super.getMaximumSize().width, getPreferredSize().height);
	}

	@Override
	@SuppressWarnings("unchecked")
	public I getSelectedItem() {
		return (I)super.getSelectedItem();
	}

	/**
	 * Returns the selected item as string,
	 * or @c null if combo box is not in editable mode.
	 */
	public String getText() {
		return isEditable() ? getTextField().getText() : null;
	}

	/**
	 * Sets the selected item text to @p value.
	 * Combo box must be in editable mode.
	 */
	public void setText(final String value) {
		if (isEditable())
			getTextField().setText(value);
	}

	/**
	 * Returns the text field component associated with this combo box.
	 */
	public JTextField getTextField() {
		return (JTextField)getEditor().getEditorComponent();
	}

	public Iterator<I> iterator() {
		return new MComboBoxIterator();
	}

	/**
	 * A mouse wheel event handler.
	 * It selects the next/previous item in the list.
	 * @param e A mouse wheel event
	 */
	public void mouseWheelMoved(final MouseWheelEvent e) {
		int count = getItemCount();

		if (count < 2)
			return;

		int index = getSelectedIndex();
		if (index == -1)
			index = 0;

		// select previous item
		if (e.getWheelRotation() < 0) {
			if (index > 0)
				setSelectedIndex(index - 1);
		}
		// select next item
		else {
			if (index < count - 1)
				setSelectedIndex(index + 1);
		}
	}

	/**
	 * Connects @p action with the @c VK_ENTER key.
	 * The action is executed only if the text field has focus.
	 */
	public void setEnterPressAction(final Action action) {
		action.putValue(Action.NAME, "MComboBox.setEnterPressAction");
		UI.connect(getTextField(), UI.WHEN_FOCUSED, VK_ENTER, action);
	}

	// protected

	protected void onSelect() { }

	// public classes

	/** A combo box iterator. */
	public final class MComboBoxIterator extends AbstractIterator<I> {

		// public

		/**
		 * Returns object at @p index.
		 */
		@SuppressWarnings("unchecked")
		public I getObjectAt(final int index) {
			return (I)getItemAt(index);
		}

		/**
		 * Returns the number of objects.
		 */
		public int getObjectCount() {
			return getItemCount();
		}

	}

}
