// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.common;

import java.awt.Component;
import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.UIManager;
import javax.swing.filechooser.FileSystemView;

/** A file chooser. */
public class MFileChooser extends JFileChooser {

	// private

	private static boolean initDone;
	private Component parent;
	private String configKey;

	// public

	/**
	 * Constructs a new file chooser.
	 * @param parent A parent dialog
	 * @param title A dialog title
	 */
	public MFileChooser(final MDialog parent, final String title) {
		init(parent, title);
	}

	/**
	 * Constructs a new file chooser.
	 * @param parent A parent frame
	 */
	public MFileChooser(final MFrame parent) {
		this(parent, null);
	}

	/**
	 * Constructs a new file chooser.
	 * @param parent A parent frame
	 * @param title A dialog title
	 */
	public MFileChooser(final MFrame parent, final String title) {
		init(parent, title);
	}

	/**
	 * Adds a new filter to the filter combo box (<b>Files of Type</b>).
	 * EXAMPLE:
	 * @code
	 * ...
	 * fileChooser.addFilter(".zip", _("Zip Archive")); // filter *.zip files
	 * ...
	 * @endcode
	 * @param types A file types to display
	 * @param description A file types description
	 */
	public void addFilter(final String types, final String description) {
		addChoosableFileFilter(new MFileFilter(types, description));
	}

	/**
	 * Returns the current directory.
	 */
	public String getCurrentDir() {
		return getCurrentDirectory().getPath();
	}

	/**
	 * Sets the current directory to @p value.
	 */
	public void setCurrentDir(final String value) {
		setCurrentDirectory((value == null) ? null : new File(value));
	}

	/**
	 * Returns the selected path (file name).
	 */
	public String getSelectedPath() {
		return getSelectedFile().getPath();
	}

	/**
	 * Sets the selected path to @p value.
	 */
	public void setSelectedPath(final String value) {
		setSelectedFile(new File(value));
	}

	public void setConfigKey(final String value) {
		if (TK.isEmpty(value))
			return;

		configKey = "fileChooser." + value + ".dir";
		setCurrentDir(Config.getGlobal().read(configKey, FS.getUserDir()));
	}

	/**
	 * Shows the "open" dialog, and returns @c true if approved.
	 */
	public boolean openDialog() {
		if (showOpenDialog(parent) == APPROVE_OPTION) {
			if (configKey != null)
				Config.getGlobal().write(configKey, getCurrentDir());

			return true;
		}

		return false;
	}

	/**
	 * Shows the "save" dialog, and returns @c true if approved.
	 */
	public boolean saveDialog() {
		if (showSaveDialog(parent) == APPROVE_OPTION) {
			if (configKey != null)
				Config.getGlobal().write(configKey, getCurrentDir());

			return true;
		}

		return false;
	}

	public void setTitle(final String value) {
		setDialogTitle(value);
	}

	@Override
	public void setup(final FileSystemView view) {
		// init custom icons
		if (!initDone) {
			if (UI.customIcons.get()) {
				final int FILE_CHOOSER_ICON_SIZE = 16;
				UIManager.put("FileChooser.detailsViewIcon", FS.getIcon("ui/detailedview", FILE_CHOOSER_ICON_SIZE));
				UIManager.put("FileChooser.homeFolderIcon", FS.getIcon("ui/home", FILE_CHOOSER_ICON_SIZE));
				UIManager.put("FileChooser.listViewIcon", FS.getIcon("ui/multicolumnview", FILE_CHOOSER_ICON_SIZE));
				UIManager.put("FileChooser.newFolderIcon", FS.getIcon("ui/newfolder", FILE_CHOOSER_ICON_SIZE));
				UIManager.put("FileChooser.upFolderIcon", FS.getIcon("ui/up", FILE_CHOOSER_ICON_SIZE));
// TODO: 2.0: UIManager.put("FileView.computerIcon", FS.getIcon("ui/computer"));
				final int FILE_VIEW_ICON_SIZE = 16;
				UIManager.put("FileView.directoryIcon", FS.getIcon("ui/folder", FILE_VIEW_ICON_SIZE));
				UIManager.put("FileView.fileIcon", FS.getIcon("ui/file", FILE_VIEW_ICON_SIZE));
				UIManager.put("FileView.floppyDriveIcon", FS.getIcon("ui/floppy", FILE_VIEW_ICON_SIZE));
				UIManager.put("FileView.hardDriveIcon", FS.getIcon("ui/hdd", FILE_VIEW_ICON_SIZE));
			}
			initDone = true;
		}
		super.setup(view);
	}

	// private

	private void init(final Component parent, final String title) {
		this.parent = parent;
		setFileHidingEnabled(true);
		setTitle(title);
	}

}
