// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.common;

import static java.lang.System.err;

import java.util.logging.*;

/** A logger. */
public final class MLogger {

	// private

	private static Logger logger;

	// static

	/**
	 * Initializes the logger.
	 */
	static {
		logger = Logger.getLogger("log");
		logger.addHandler(new MHandler());
		logger.setLevel(Level.ALL);
		logger.setUseParentHandlers(false);
	}

	// public

	/**
	 * Logs an error text.
	 * @param text A text
	 */
	public static void error(final String text) {
		logger.severe(text);
	}

	/**
	 * Logs an error text.
	 * @param text A text
	 * @param args Arguments used to format text
	 */
	public static void error(final String text, final Object... args) {
		logger.severe(String.format(text, args));
	}

	/**
	 * Logs an exception error
	 * and prints a stack trace to @b stderr.
	 * @param exception A throwable exception
	 */
	public static void exception(final Throwable exception) {
		if (exception == null)
			return;

		error(exception.toString());
		exception.printStackTrace(err);
	}

	/**
	 * Logs an info text.
	 * @param text A text
	 */
	public static void info(final String text) {
		logger.info(text);
	}

	/**
	 * Logs an info text.
	 * @param text A text
	 * @param args Arguments used to format text
	 */
	public static void info(final String text, final Object... args) {
		logger.info(String.format(text, args));
	}

	public static void trace() {
		new Throwable().printStackTrace(err);
	}

	/**
	 * Logs a warning text.
	 * @param text A text
	 */
	public static void warning(final String text) {
		logger.warning(text);
	}

	/**
	 * Logs a warning text.
	 * @param text A text
	 * @param args Arguments used to format text
	 */
	public static void warning(final String text, final Object... args) {
		logger.warning(String.format(text, args));
	}

	/**
	 * Logs a temporary debug text.
	 * The text is prefixed with <b>***:</b> string.
	 * The <b>XX</b> string is easy to find and remove from the source code.
	 * @param text A text
	 */
	public static void XXX(final String text) {
		logger.finest("***: " + text);
	}

	/**
	 * Logs a temporary debug text.
	 * The text is prefixed with <b>***:</b> string.
	 * The <b>XX</b> string is easy to find and remove from the source code.
	 * @param text A text
	 * @param args Arguments used to format text
	 */
	public static void XXX(final String text, final Object... args) {
		XXX(String.format(text, args));
	}

	/**
	 * Prints a temporary debug text to @b stderr.
	 * This function also prints the current stack trace.
	 * The text is prefixed with <b>***:</b> string.
	 * The <b>XX</b> string is easy to find and remove from the source code.
	 * @param text A text
	 */
	public static void XXXTrace(final String text) {
		XXX(text);
		trace();
	}

	// private classes

	private static final class MFormatter extends Formatter {

		// public

		public String format(LogRecord record) {
			return
				record.getLevel() + ": " + DT.format(DT.now(), DT.MEDIUM, DT.MEDIUM) + '\n' +
				record.getMessage() + '\n';
		}

	}

	private static final class MHandler extends Handler {

		// private

		private MFormatter formatter = new MFormatter();

		// public

		public void close() { }

		public void flush() { }

		public void publish(LogRecord record) {
			err.println(formatter.format(record));
		}

	}

}
