// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.common;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.LayoutManager;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;

/** A panel. */
public class MPanel extends JPanel implements MBorderLayout {

	// public

	public static final LayoutManager NO_LAYOUT = null;

	// private

	private int boxLayoutAxis;

	// public

	/**
	 * Constructs a new panel with layout @p layout.
	 */
	public MPanel(final LayoutManager layout) {
		super(layout);
	}

	public void add(final JComponent component) {
		if (component != null) {
			if ((boxLayoutAxis == BoxLayout.Y_AXIS) && (getLayout() instanceof BoxLayout))
				component.setAlignmentX(LEFT_ALIGNMENT);
			super.add(component);
		}
	}

	/**
	 * Adds a horizontal or vertical gap.
	 * @b NOTE: This works only with vertical/horizontal box layout.
	 * @see createHBoxPanel
	 */
	public void addGap() {
		add(
			(boxLayoutAxis == BoxLayout.X_AXIS)
			? Box.createRigidArea(new Dimension(5, 0))
			: Box.createRigidArea(new Dimension(0, 5))
		);
	}

	/**
	 * Adds a horizontal or vertical stretch.
	 * @b NOTE: This works only with vertical/horizontal box layout.
	 * @see createHBoxPanel
	 */
	public void addStretch() {
		add(
			(boxLayoutAxis == BoxLayout.X_AXIS)
			? Box.createHorizontalGlue()
			: Box.createVerticalGlue()
		);
	}

	/**
	 * Constructs and returns a new panel with @b border layout.
	 */
	public static MPanel createBorderPanel() {
		return new MPanel();
	}

	/**
	 * Constructs and returns a new panel with @b border layout.
	 * @param gap A horizontal and vertical gap
	 */
	public static MPanel createBorderPanel(final int gap) {
		return new MPanel(gap, gap);
	}

	/**
	 * Constructs and returns a new panel with @b grid layout.
	 * Defaults:
	 * - Number of columns: @c 1
	 * - Horizontal and vertical gap size: @c 0
	 * @param rows A number of rows
	 */
	public static MPanel createGridPanel(final int rows) {
		return new MPanel(rows, 1, 0, 0);
	}

	/**
	 * Constructs and returns a new panel with @b grid layout.
	 * Horizontal and vertical gap size is @c 0.
	 * @param rows A number of rows
	 * @param cols A number of columns
	 */
	public static MPanel createGridPanel(final int rows, final int cols) {
		return new MPanel(rows, cols, 0, 0);
	}

	/**
	 * Constructs and returns a new panel with @b grid layout.
	 * @param rows A number of rows
	 * @param cols A number of columns
	 * @param hgap A horizontal gap
	 * @param vgap A vertical gap
	 */
	public static MPanel createGridPanel(final int rows, final int cols, final int hgap, final int vgap) {
		return new MPanel(rows, cols, hgap, vgap);
	}

	public static MPanel createHBoxPanel() {
		MPanel p = new MPanel(NO_LAYOUT);
		p.setHBoxLayout();

		return p;
	}

	/**
	 * Constructs and returns a new panel with @b border layout.
	 * @param text A border title
	 */
	public static MPanel createTitledPanel(final String text) {
		return new MPanel(text);
	}

	public static MPanel createVBoxPanel() {
		MPanel p = new MPanel(NO_LAYOUT);
		p.setVBoxLayout();
		p.setAlignmentY(TOP_ALIGNMENT);

		return p;
	}

	public void setHBoxLayout() {
		boxLayoutAxis = BoxLayout.X_AXIS;
		setLayout(new BoxLayout(this, boxLayoutAxis));
	}

	/**
	 * Sets margin border with @p margin size.
	 */
	public void setMargin(final int margin) {
		setMargin(margin, margin, margin, margin);
	}

	public void setMargin(final int topMargin, final int leftMargin, final int bottomMargin, final int rightMargin) {
		setBorder(BorderFactory.createEmptyBorder(topMargin, leftMargin, bottomMargin, rightMargin));
	}

	/**
	 * Sets border title to @p value.
	 */
	public void setTitle(final String value) {
		setBorder(BorderFactory.createTitledBorder(value));
	}

	public void setVBoxLayout() {
		boxLayoutAxis = BoxLayout.Y_AXIS;
		setLayout(new BoxLayout(this, boxLayoutAxis));
	}

	// MBorderLayout

	public void addCenter(final JComponent component) {
		UI.addCenter(this, component);
	}

	public void addEast(final JComponent component) {
		add(component, BorderLayout.EAST);
	}

	public void addNorth(final JComponent component) {
		add(component, BorderLayout.NORTH);
	}

	public void addSouth(final JComponent component) {
		add(component, BorderLayout.SOUTH);
	}

	public void addWest(final JComponent component) {
		add(component, BorderLayout.WEST);
	}

	// protected

	protected MPanel() {
		super(new BorderLayout());
	}

	protected MPanel(final String text) {
		this(new BorderLayout());
		setTitle(text);
	}

	protected MPanel(final int hgap, final int vgap) {
		this(new BorderLayout(hgap, vgap));
	}

	protected MPanel(final int rows, final int cols, final int hgap, final int vgap) {
		this(new GridLayout(rows, cols, hgap, vgap));
	}

}
