// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.common;

import static java.awt.event.KeyEvent.*;

import static org.makagiga.common.UI._;

import java.awt.Insets;
import java.awt.event.MouseEvent;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.JTextComponent;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;

/** A text component extension. */
public final class MText {

	// public

	public static final int MARGIN = 2;

	public static final String UNDO = "MText.undo"; /**< Undo action. */
	public static final String REDO = "MText.redo"; /**< Redo action. */
	public static final String CUT = "MText.cut"; /**< Cut action. */
	public static final String COPY = "MText.copy"; /**< Copy action. */
	public static final String PASTE = "MText.paste"; /**< Paste action. */
	public static final String CLEAR = "MText.clear"; /**< Clear action. */
	public static final String DELETE = "MText.delete"; /**< Delete action. */
	public static final String SELECT_ALL = "MText.selectAll"; /**< Select all action. */

	// private

	private static final String UNDO_MANAGER = "MText.undoManager";

	// public

	/**
	 * Installs common actions, enables drag and drop, and sets margin.
	 * @param textComponent A non-null text component
	 */
	public static void commonSetup(final JTextComponent textComponent) {
		textComponent.setDragEnabled(true);
		textComponent.setMargin(new Insets(MARGIN, MARGIN, MARGIN, MARGIN));
		install(textComponent);
	}

	/**
	 * Creates and returns menu for text component (cut, copy, paste, etc).
	 * @param textComponent A non-null text component
	 */
	public static MMenu createMenu(final JTextComponent textComponent) {
		updateActions(textComponent);

		MMenu menu = new MMenu();
		if (textComponent.isEditable()) {
			menu.add(getAction(textComponent, UNDO));
			if (OS.isKDE())
				menu.add(getAction(textComponent, REDO));
			menu.addSeparator();
			menu.add(getAction(textComponent, CUT));
			menu.add(getAction(textComponent, COPY));
			menu.add(getAction(textComponent, PASTE));
			menu.add(getAction(textComponent, OS.isKDE() ? CLEAR : DELETE));
			menu.addSeparator();
		}
		else {
			menu.add(getAction(textComponent, COPY));
		}
		menu.add(getAction(textComponent, SELECT_ALL));

		return menu;
	}

	/**
	 * Returns a text component action.
	 * @param textComponent A non-null text component
	 * @param name An action name
	 */
	public static AppAction getAction(final JTextComponent textComponent, final String name) {
		return (AppAction)textComponent.getClientProperty(name);
	}

	/**
	 * Installs actions and undo manager.
	 * @param textComponent A non-null text component
	 */
	public static void install(final JTextComponent textComponent) {
		// caret listener
		textComponent.addCaretListener(new CaretListener() {
			public void caretUpdate(final CaretEvent e) {
				updateSelectionActions(textComponent);
			}
		} );

		// context menu
		textComponent.addMouseListener(new MMouseAdapter() {
			@Override
			public void popupTrigger(final MouseEvent e) {
				JTextComponent textComponent = (JTextComponent)e.getSource();
				createMenu(textComponent).showPopup(textComponent, e.getX(), e.getY());
			}
		} );

		installUndoManager(textComponent);

		// undo
		AppAction undoAction = new AppAction(_("&Undo"), "ui/undo", VK_Z, CTRL_MASK) {
			public void onAction() {
				try {
					getUndoManager(textComponent).undo();
				}
				catch (CannotUndoException exception) {
					MLogger.exception(exception);
				}
				updateActions(textComponent);
			}
		};
		undoAction.setHTMLHelp(_("Undoes the last operation."));

		// redo
		AppAction redoAction = new AppAction(_("&Redo"), "ui/redo", VK_Y, CTRL_MASK) {
			public void onAction() {
				try {
					getUndoManager(textComponent).redo();
				}
				catch (CannotRedoException exception) {
					MLogger.exception(exception);
				}
				updateActions(textComponent);
			}
		};
		redoAction.setHTMLHelp(_("Redoes the last operation."));

		// cut
		AppAction cutAction = new AppAction(_("Cu&t"), "ui/cut", VK_X, CTRL_MASK) {
			public void onAction() {
				textComponent.cut();
			}
		};
		cutAction.setHTMLHelp(_("Cuts the selected text to the clipboard."));

		// copy
		AppAction copyAction = new AppAction(_("&Copy"), "ui/copy", VK_C, CTRL_MASK) {
			public void onAction() {
				textComponent.copy();
			}
		};
		copyAction.setHTMLHelp(_("Copies the selected text to the clipboard."));

		// paste
		AppAction pasteAction = new AppAction(_("&Paste"), "ui/paste", VK_V, CTRL_MASK) {
			public void onAction() {
// FIXME: 2.0: paste "from" Kate (KDE)
// FIXME: 2.0: paste "from" Firefox
				textComponent.paste();
			}
		};
		pasteAction.setHTMLHelp(_("Pastes from the clipboard."));

		// clear
		AppAction clearAction = new AppAction(_("C&lear"), "ui/clear", VK_U, CTRL_MASK) {
			public void onAction() {
				textComponent.setText(null); // clear
			}
		};
		clearAction.setHTMLHelp(_("Clears all text."));

		// delete
		AppAction deleteAction = new AppAction(_("&Delete"), "ui/delete") {
			public void onAction() {
				textComponent.replaceSelection(null); // delete selected text
			}
		};
		deleteAction.setHTMLHelp(_("Deletes the selected text."));

		// select all
		AppAction selectAllAction = new AppAction(_("Select &All"), null, VK_A, CTRL_MASK) {
			public void onAction() {
				selectAll(textComponent);
			}
		};
		selectAllAction.setHTMLHelp(_("Selects all text."));

		// ctrl+z - undo
		UI.connect(textComponent, UI.WHEN_FOCUSED, VK_Z, AppAction.CTRL_MASK, undoAction);
		// ctrl+y - redo
		UI.connect(textComponent, UI.WHEN_FOCUSED, VK_Y, AppAction.CTRL_MASK, redoAction);
		// ctrl+u - clear
		UI.connect(textComponent, UI.WHEN_FOCUSED, VK_U, AppAction.CTRL_MASK, clearAction);
		// shift+del - cut
		UI.connect(textComponent, UI.WHEN_FOCUSED, VK_DELETE, AppAction.SHIFT_MASK, cutAction);
		// ctrl+ins - copy
		UI.connect(textComponent, UI.WHEN_FOCUSED, VK_INSERT, AppAction.CTRL_MASK, copyAction);
		// shift+ins - paste
		UI.connect(textComponent, UI.WHEN_FOCUSED, VK_INSERT, AppAction.SHIFT_MASK, pasteAction);
		textComponent.putClientProperty(UNDO, undoAction);
		textComponent.putClientProperty(REDO, redoAction);
		textComponent.putClientProperty(CUT, cutAction);
		textComponent.putClientProperty(COPY, copyAction);
		textComponent.putClientProperty(PASTE, pasteAction);
		textComponent.putClientProperty(CLEAR, clearAction);
		textComponent.putClientProperty(DELETE, deleteAction);
		textComponent.putClientProperty(SELECT_ALL, selectAllAction);
	}

	/**
	 * Installs undo manager.
	 * @param textComponent A non-null text component
	 */
	public static void installUndoManager(final JTextComponent textComponent) {
		textComponent.getDocument().addUndoableEditListener(new UndoableEditListener() {
			public void undoableEditHappened(final UndoableEditEvent e) {
				if (textComponent.isEditable())
					getUndoManager(textComponent).addEdit(e.getEdit());
				updateActions(textComponent);
			}
		} );
		textComponent.putClientProperty(UNDO_MANAGER, new UndoManager());
	}

	// private

	private static UndoManager getUndoManager(final JTextComponent textComponent) {
		return (UndoManager)textComponent.getClientProperty(UNDO_MANAGER);
	}

	private static boolean hasSelection(final JTextComponent textComponent) {
		int i = textComponent.getCaretPosition();

		return
			(textComponent.getSelectionEnd() != i) ||
			(textComponent.getSelectionStart() != i);
	}

	private static boolean isEmpty(final JTextComponent textComponent) {
		return TK.isEmpty(textComponent.getText());
	}

	private static void selectAll(final JTextComponent textComponent) {
		textComponent.requestFocusInWindow();
		textComponent.selectAll();
	}

	private static void updateActions(final JTextComponent textComponent) {
		UndoManager undoManager = getUndoManager(textComponent);
		boolean isEditable = textComponent.isEditable();
		boolean isEmpty = isEmpty(textComponent);
		getAction(textComponent, UNDO).setEnabled(isEditable && undoManager.canUndo());
		getAction(textComponent, REDO).setEnabled(isEditable && undoManager.canRedo());
		getAction(textComponent, PASTE).setEnabled(isEditable);
		getAction(textComponent, CLEAR).setEnabled(isEditable && !isEmpty);
		getAction(textComponent, SELECT_ALL).setEnabled(!isEmpty);
		updateSelectionActions(textComponent);
	}

	private static void updateSelectionActions(final JTextComponent textComponent) {
		boolean isEditable = textComponent.isEditable();
		boolean isSelection = MText.hasSelection(textComponent);
		getAction(textComponent, CUT).setEnabled(isEditable && isSelection);
		getAction(textComponent, COPY).setEnabled(isSelection);
		getAction(textComponent, DELETE).setEnabled(isEditable && isSelection);
	}

}
