// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.common;

import static java.awt.Color.*;

import java.awt.Color;
import java.awt.Insets;
import java.net.URL;
import javax.swing.JTextPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.Document;

/**
 * A text pane.
 * Hyperlink events are handled by default. See @ref onHyperlinkClick.
 */
public class MTextPane extends JTextPane {

	// public

	public static final String TEXT_HTML_UTF_8 = "text/html; charset=UTF-8"; /**< Content type and encoding. */

	// public

	/**
	 * Constructs a new text pane.
	 */
	public MTextPane() {
		setDragEnabled(true);
		MText.install(this);
		addHyperlinkListener(new HyperlinkListener() {
			public void hyperlinkUpdate(final HyperlinkEvent e) {
				if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
					String d = e.getDescription();
					URL url = e.getURL();
					// <a name="name"></a>
					if (d.startsWith("#")) {
						scrollToReference(d.substring(1));
					}
					// external url
					else if (!onHyperlinkClick(e)) {
						OS.launchBrowser(url);
					}
				}
				else if (e.getEventType() == HyperlinkEvent.EventType.ENTERED) {
					String description = e.getDescription();
					MStatusBar.setText(description);
					setToolTipText(
						UI.toolTipsDetails.get()
						? UI.makeHTML(description)
						: null
					);
				}
				else if (e.getEventType() == HyperlinkEvent.EventType.EXITED) {
					MStatusBar.ready();
					setToolTipText(null);
				}
			}
		} );
	}

	/**
	 * Removes all text.
	 */
	public synchronized void clear() {
		setText(null);
	}

	/**
	 * Returns a plain text (no HTML formatting), or @c null if error.
	 */
	public synchronized String getPlainText() {
		try {
			Document doc = getDocument();

			return doc.getText(0, doc.getLength());
		}
		catch (Exception exception) {
			MLogger.exception(exception);

			return null;
		}
	}

	/**
	 * Requests focus in window, then selects all text.
	 */
	@Override
	public synchronized void selectAll() {
		requestFocusInWindow();
		super.selectAll();
	}

	/**
	 * Sets background color to @p value.
	 * Caret and selection colors are also adjusted.
	 */
	public synchronized void setColors(final Color value) {
		if (value == null) {
			setCaretColor(BLACK);
			setBackground(WHITE);
			setSelectedTextColor(WHITE);
			setSelectionColor(BLACK);
		}
		else {
// !!!: HTML background color
/*
			getKit().getStyleSheet().addRule(
				String.format(
					"body { background-color: %s; }",
					TK.toHTMLTriplet(value)
				)
			);
*/
			Color xorColor = UI.getXorColor(value);
			setCaretColor(xorColor);
			setBackground(value);
			setSelectedTextColor(UI.getXorColor(xorColor)); // double xor
			setSelectionColor(xorColor);
		}
	}

	/**
	 * Clears the stream description property.
	 */
	public synchronized void setForceReload() {
		getDocument().putProperty(Document.StreamDescriptionProperty, null);
	}

	/**
	 * Sets margin to @c 0.
	 */
	public synchronized void setNoMargin() {
		setMargin(new Insets(0, 0, 0, 0));
	}

	// protected

	/**
	 * Invoked when user clicked on the hyperlink.
	 * @param e A hyperlink event
	 */
	protected synchronized boolean onHyperlinkClick(final HyperlinkEvent e) { return false; }

}
