// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.common;

import javax.swing.JTree;
import javax.swing.ToolTipManager;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

/** A tree. */
public class MTree<I extends MTreeItem> extends JTree {

	// private

	private DefaultTreeModel model;
	private I rootItem;

	// public

	/**
	 * Constructs a new tree.
	 * Defaults:
	 * - Tool tip manager will register this component
	 * - The model is @c DefaultTreeModel
	 * - The row height is -1 (auto size)
	 */
	public MTree() {
		ToolTipManager.sharedInstance().registerComponent(this);
		model = new DefaultTreeModel(null);
		setModel(model);
		setRowHeight(-1);
	}

	/**
	 * Adds a new item to the tree.
	 * @param parent A parent item
	 * @param item A new item
	 */
	public void addItem(final I parent, final I item) {
		model.insertNodeInto(item, parent, parent.getChildCount());
	}

	/**
	 * Removes @p item from the tree.
	 */
	public void deleteItem(final I item) {
		model.removeNodeFromParent(item);
	}

	/**
	 * Expands @p item.
	 */
	public void expandItem(final I item) {
		expandPath(item.getTreePath());
	}

	/**
	 * Focuses @p item.
	 * @see selectItem
	 */
	public void focusItem(final I item) {
		if (item == null)
			return;

		selectItem(item);
		scrollPathToVisible(item.getTreePath());
	}

	/**
	 * Returns an item at row @p index, or @c null if item was not found.
	 */
	@SuppressWarnings("unchecked")
	public I getItemAtRow(final int index) {
		TreePath path = getPathForRow(index);

		if (path == null)
			return null;

		I item = (I)path.getLastPathComponent();

		return item;
	}

	/**
	 * Sets root item to @p item.
	 */
	public void setRootItem(final I item) {
		rootItem = item;
		model.setRoot(rootItem);
	}

	/**
	 * Returns a selected item, or @c null if no selection.
	 */
	@SuppressWarnings("unchecked")
	public I getSelectedItem() {
		TreePath path = getSelectionPath();

		if (path == null)
			return null;

		return (I)path.getLastPathComponent();
	}

	/**
	 * Selects @p item.
	 * @see focusItem
	 */
	public void selectItem(final I item) {
		if (item != null)
			setSelectionPath(item.getTreePath());
	}

}
