// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.common;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;

/** An abstract markup builder. */
public abstract class MarkupBuilder {

	// protected

	protected boolean xml;
	protected int indent;
	protected String tagEnd;

	/**
	 * A buffer
	 */
	protected StringBuilder builder;

	// public

	/**
	 * Constructs a new markup builder.
	 * The string builder buffer size is set to @c 4KB.
	 */
	public MarkupBuilder(final boolean xml) {
		builder = new StringBuilder(4096);
		this.xml = xml;
		tagEnd = xml ? " />" : ">";
	}

	/**
	 * Adds a comment with @p text.
	 * @code
	 * ...
	 * builder.addComment("This is a comment");
	 * // <!-- This is a comment -->\n
	 * ...
	 * @endcode
	 */
	public void addComment(final String text) {
		appendLine("<!-- %s -->", text);
	}

	/**
	 * Appends @p code to the buffer.
	 */
	public void append(final String code) {
		builder.append(getIndent() + code);
	}

	public void appendChars(final String name, final String chars) {
		appendLine("<" + name + ">" + TK.escapeXML(chars) + "</" + name + ">");
	}

	/**
	 * Appends @p code + <b>new line</b> to the buffer.
	 */
	public void appendLine(final String code) {
		builder.append(getIndent() + code + '\n');
	}

	/**
	 * Appends code + <b>new line</b> to the buffer.
	 * @param format A code to append
	 * @param args A @p format arguments
	 */
	public void appendLine(final String format, final Object... args) {
		appendLine(String.format(format, args));
	}

	/**
	 * Begins a tag.
	 * @code
	 * ...
	 * builder.beginTag("b"); // <b>\n
	 * ...
	 * @endcode
	 */
	public void beginTag(final String name) {
		appendLine("<" + name + ">");
		indent++;
	}

	public void beginTag(final String name, final String... attrs) {
		StringBuilder s = new StringBuilder(256);
		s.append("<" + name);
		if (attrs != null) {
			boolean attrName = true;
			for (String a : attrs) {
				if (attrName) {
					s.append(" " + a + "=\"");
					attrName = false;
				}
				else {
					s.append(a + "\"");
					attrName = true;
				}
			}
		}
		appendLine(s.toString() + ">");
		indent++;
	}

	/**
	 * Ends a tag.
	 * @code
	 * ...
	 * builder.endTag("b"); // </b>\n
	 * ...
	 * @endcode
	 */
	public void endTag(final String name) {
		decIndent();
		appendLine("</" + name + ">");
	}

	public String getIndent() {
		if (indent == 0)
			return "";

		StringBuilder s = new StringBuilder(indent);
		for (int i = 0; i < indent; i++)
			s.append("\t");

		return s.toString();
	}

	/**
	 * Saves buffer to the specified file.
	 * The text is saved in the UTF-8 encoding.
	 * @param path A destination file name
	 */
	public void saveToFile(final String path) throws IOException {
		BufferedWriter writer = null;
		try {
			writer = FS.getUTF8Writer(path);
			writer.write(toString());
		}
		finally {
			FS.close(writer);
		}
	}

	public void saveToStream(final OutputStream stream) throws IOException {
		stream.write(builder.toString().getBytes(), 0, builder.length());
	}

	public void singleTag(final String name, final String... attrs) {
		StringBuilder s = new StringBuilder(256);
		s.append("<" + name);
		if (attrs != null) {
			boolean attrName = true;
			for (String a : attrs) {
				if (attrName) {
					s.append(" " + a + "=\"");
					attrName = false;
				}
				else {
					s.append(a + "\"");
					attrName = true;
				}
			}
		}
		appendLine(s.toString() + tagEnd);
	}

	/**
	 * Returns the markup code.
	 */
	@Override
	public String toString() {
		return builder.toString();
	}

	// protected

	protected void decIndent() {
		if (indent > 0)
			indent--;
	}

}
