// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.common;

/**
 * A property.
 *
 * EXAMPLE:
 * @code
 * ...
 * public final Property<MyClass> attr = new Property<MyClass>();
 * ...
 * attr.set(someValueOfTypeMyClass);
 * ...
 * System.out.println("Value = " + attr);
 * ...
 * processMyValue(attr.get());
 * ...
 * @endcode
 */
public class Property<T> {

	// protected

	protected T value;

	// public

	/**
	 * Constructs a new property with @c null value.
	 */
	public Property() {
		this(null);
	}

	/**
	 * Constructs a new property with value @p value.
	 */
	public Property(final T value) {
		this.value = value;
	}

	/**
	 * Sets value to @c null.
	 */
	public void clear() {
		value = null;
	}

	/**
	 * Returns @c true if value equals @p o.
	 */
	@Override
	public boolean equals(final Object o) {
		return (value == null) ? false : value.equals(o);
	}

	/**
	 * Returns the value.
	 */
	public T get() { return value; }

	/**
	 * Sets value to @p value.
	 */
	public void set(final T value) { this.value = value; }

	@Override
	public int hashCode() {
		return (value == null) ? 0 : value.hashCode();
	}

	/**
	 * Returns @c true if value is @c null.
	 */
	public boolean isNull() {
		return value == null;
	}

	/**
	 * Returns a @c String representation of the value,
	 * or @c null if @c value is @c null.
	 */
	@Override
	public String toString() {
		return (value == null) ? null : value.toString();
	}

}
