// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.common;

import java.awt.Color;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.*;

/** Low-level tool kit. */
public final class TK {

	// public

	/**
	 * Returns @c true if @p string contains @p substring.
	 * Case is ignored.
	 */
	public static boolean containsIgnoreCase(final String string, final String substring) {
		if ((string == null) || (substring == null))
			return false;

		return string.toUpperCase().indexOf(substring.toUpperCase()) != -1;
	}

	/**
	 * Raises @c NullPointerException.
	 * @code
	 * Object o = null;
	 * o.toString();
	 * @endcode
	 */
	public static void crash() {
		Object o = null;
		o.toString();
	}

	/**
	 * Converts @p value to URL. (e.g. " " ->  "%20")
	 */
	public static String escapeURL(final String value) {
		try {
			return URLEncoder.encode(value, "UTF-8");
		}
		catch (UnsupportedEncodingException exception) {
			MLogger.exception(exception);

			return value;
		}
	}

	/**
	 * Escapes XML. (e.g. "<" ->  "&lt;")
	 * @param value A value to escape
	 */
	public static String escapeXML(final String value) {
		if (value == null)
			return null;

		int len = value.length();

		if (len == 0)
			return value;

		char c;
		StringBuilder buf = new StringBuilder(len * 2);
		for (int i = 0; i < len; i++) {
			c = value.charAt(i);
			switch (c) {
				case '<':
					buf.append("&lt;");
					break;
				case '>':
					buf.append("&gt;");
					break;
				case '&':
					buf.append("&amp;");
					break;
				case '\'':
					buf.append("&apos;");
					break;
				case '\"':
					buf.append("&quot;");
					break;
				default:
					buf.append((c > 128) ? ("&#" + (int)c + ";") : c);
					break;
			}
		}

		return buf.toString();
	}

	/**
	 * Returns @c true if the @p value is @c null or empty.
	 */
	public static boolean isEmpty(final String value) {
		return (value == null) || (value.length() == 0);
	}

	/**
	 * Returns @c true if the @p value array is @c null or empty.
	 */
	public static boolean isEmpty(final Object[] value) {
		return (value == null) || (value.length == 0);
	}

	/**
	 * Joins a collection into one string.
	 * Items are separated by @p delimeter.
	 * @return @c null if @p value is @c null
	 */
	public static String join(final Collection value, final String delimeter) {
		if (value == null)
			return null;

		boolean wasFirst = false;
		StringBuilder buf = new StringBuilder(1024);
		Iterator i = value.iterator();
		while (i.hasNext()) {
			buf.append(wasFirst ? (delimeter + i.next()) : i.next());
			wasFirst = true;
		}

		return buf.toString();
	}

	/**
	 * Joins an array of @p values into one string.
	 * Items are separated by @p delimeter.
	 * @return @c null if @p values is @c null
	 */
	public static String join(final Object[] values, final String delimeter) {
		if (values == null)
			return null;

		boolean wasFirst = false;
		StringBuilder buf = new StringBuilder(1024);
		for (Object i : values) {
			buf.append(wasFirst ? (delimeter + i) : i);
			wasFirst = true;
		}

		return buf.toString();
	}

	/**
	 * Quits the application with exit code @b 0.
	 */
	public synchronized static void quit() {
		quit(0);
	}

	/**
	 * Quits the application with the exit code equal @p status.
	 */
	public synchronized static void quit(final int status) {
		MLogger.info("QUIT: " + status);
		System.exit(status);
	}

	/**
	 * Sorts @c Hashtable by keys.
	 * @param table A hashtable to sort
	 * @param comparator A comparator used to compare key values
	 * @return Sorted array list
	 */
	public static <K> ArrayList<K> sortByKey(final Hashtable<K, ?> table, final Comparator<K> comparator) {
		ArrayList<K> sorted = new ArrayList<K>(table.size());
		Set<K> keys = table.keySet();
		for (K i : keys)
			sorted.add(i);
		Collections.sort(sorted, comparator);

		return sorted;
	}

	/**
	 * Sorts @c Hashtable by values.
	 * @param table A hashtable to sort
	 * @param comparator A comparator used to compare value values
	 * @return Sorted array list
	 */
	public static <V> ArrayList<V> sortByValue(final Hashtable<?, V> table, final Comparator<V> comparator) {
		ArrayList<V> sorted = new ArrayList<V>(table.values());
		Collections.sort(sorted, comparator);

		return sorted;
	}

	/**
	 * Converts @p color to HTML triplet.
	 * If @p color is @c null, the "#000000" is returned.
	 *
	 * EXAMPLE:
	 * @code
	 * ...
	 * String htmlColor = TK.toHTMLTriplet(Color.RED); // htmlColor = "#ff0000"
	 * ...
	 * @endcode
	 */
	public static String toHTMLTriplet(final Color color) {
		if (color == null)
			return "#000000";

		return
			'#' +
			toByteHex(color.getRed()) +
			toByteHex(color.getGreen()) +
			toByteHex(color.getBlue());
	}

	/**
	 * Converts an array of objects to @c String.
	 * @param value An array of objects
	 */
	public static String toString(final Object[] value) {
		if (isEmpty(value))
			return "";

		boolean addComma = false;
		StringBuilder buf = new StringBuilder(1024);
		for (Object i : value) {
			if (addComma)
				buf.append(", ");
			buf.append(i.toString());
			addComma = true;
		}

		return buf.toString();
	}

	/**
	 * Converts an URL @p value to @c String. (e.g. "%20" -> " ")
	 */
	public static String unescapeURL(final String value) {
		try {
			return URLDecoder.decode(value, "UTF-8");
		}
		catch (UnsupportedEncodingException exception) {
			MLogger.exception(exception);

			return value;
		}
	}

	// private

	private TK() { }

	private static String toByteHex(final int value) {
		if (value == 0)
			return "00";

		if (value == 0xff)
			return "ff";

		return
			(value < 16)
			? '0' + Integer.toHexString(value)
			: Integer.toHexString(value);
	}

}
