// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.console;

import static java.awt.event.KeyEvent.*;
import static java.lang.System.err;
import static java.lang.System.out;

import static org.makagiga.common.UI._;

import java.awt.Font;
import java.io.*;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;

import org.makagiga.common.*;
import org.makagiga.plugins.PluginInfo;
import org.makagiga.plugins.PluginManager;

/** A system console. */
public final class Console extends MFrame {

	// private

	private static boolean firstShow = true;
	private static Console _instance;
	private MButton executeCommandButton;
	private MComboBox<String> commandLine;
	private MTextArea list;

	// public

	public void initCommands() {
		ConsoleCommand alias;
		// args
		Commands.register("args", new ConsoleCommand(_("Show available command line arguments")) {
			public void onCommand() {
				Args.help();
			}
		} );
		// clear
		Commands.register("clear", new ConsoleCommand(_("Clear console")) {
			public void onCommand() {
				list.clear();
			}
		} );
		// crash
		Commands.register("crash", new ConsoleCommand(_("Raise null pointer exception")) {
			public void onCommand() {
				TK.crash();
			}
		} );
		// debug
		Commands.register("debug", new ConsoleCommand(_("Show debug information")) {
			public void onCommand() {
				IO.printLine();
				IO.printLine("----[ CUT HERE ]----");
				executeCommand("version");
				Args.dump();
				executeCommand("dirs");
				System.getProperties().list(err);
				IO.printLine();
				executeCommand("set");
			}
		} );
		// dirs
		Commands.register("dirs", new ConsoleCommand(_("Show directories used by this application")) {
			public void onCommand() {
				IO.printLine("User directory: " + FS.getUserDir());
				IO.printLine("Configuration directory: " + FS.getConfigDir());
			}
		} );
		// exit, quit
		alias = new ConsoleCommand(_("Quit application")) {
			public void onCommand() {
				IO.shutDown();
				TK.quit();
			}
		};
		Commands.register("exit", alias);
		Commands.register("quit", alias);
		// flush
		Commands.register("flush", new ConsoleCommand(_("Flush internal caches")) {
			public void onCommand() {
				IO.flushLog();
				FS.IconCache.getInstance().clear();
			}
		} );
		// gc
		Commands.register("gc", new ConsoleCommand(_("Run garbage collector")) {
			public void onCommand() {
				System.gc();
			}
		} );
		// help
		Commands.register("help", new ConsoleCommand(_("Show console commands")) {
			public void onCommand() {
				Commands.show();
			}
		} );
		// set
		Commands.register("set", new ConsoleCommand(_("Show system environment variables")) {
			public void onCommand() {
				Map<String, String> env = System.getenv();
				Set<Map.Entry<String, String>> envSet = env.entrySet();
				for (Map.Entry<String, String> i : envSet)
					IO.printLine(i.getKey() + "=" + i.getValue());
			}
		} );
		// version
		Commands.register("version", new ConsoleCommand(_("Show application version")) {
			public void onCommand() {
				Args.version();
			}
		} );

		ConsolePlugin plugin;
		for (PluginInfo i : PluginManager.getInfos(ConsolePlugin.TYPE)) {
			plugin = ConsolePlugin.class.cast(i.plugin.get());
			Commands.register(plugin.getCommand(), plugin.createCommand());
		}
	}

	public static void showWindow() {
		if (_instance == null)
			_instance = new Console();

		if (firstShow) {
			firstShow = false;
			UI.centerOnScreen(_instance);
			_instance.setVisible(true);
			IO.printLine();
			IO.printLine(_("Type \"help\" to see all commands"));
			IO.printLine();
		}
		else {
			_instance.setVisible(true);
		}
		_instance.commandLine.requestFocusInWindow();
	}

	// private

	private Console() {
		super(_("Console"));
		setSize(640, 480);

		// add plugins
		ConsolePlugin.registerBuiltIn();
		if (firstShow)
			initCommands();

// MUSTANG: Font.MONOSPACED/DIALOG
		Font f = new Font("Monospaced", Font.BOLD, 14);

		// init list (center)
		list = new MTextArea();
		list.setEditable(false);
		list.setFont(f);
		if (firstShow) {
			list.append(IO.buf.toString());
			list.scrollToEnd();
		}
		addCenter(list);

		MPanel bottomPanel = MPanel.createBorderPanel();
		// init command line (south)
		commandLine = new MComboBox<String>();
		// enter - execute command
		commandLine.setEnterPressAction(new SimpleAction() {
			public void onAction() {
				executeCommandButton.doClick();
			}
		} );
		commandLine.setEditable(true);
		commandLine.setFont(f);
		commandLine.setToolTipText(UI.makeHTML(_("Enter a command.")));
		bottomPanel.addCenter(commandLine);
		executeCommandButton = new MButton(FS.getIcon("ui/ok", 16)) {
			@Override
			protected void onClick() {
				executeCommand();
			}
		};
		executeCommandButton.setToolTipText(UI.makeHTML(_("Execute command.")));
		bottomPanel.addEast(executeCommandButton);
		addSouth(bottomPanel);

		UI.connect(
			commandLine, UI.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT, VK_ESCAPE,
			new SimpleAction("Console.VK_ESCAPE") {
				public void onAction() {
					if (TK.isEmpty(commandLine.getText()))
						dispose();
					else
						commandLine.clear();
				}
			}
		);
	}

	private void executeCommand(final String command) {
		if (TK.isEmpty(command)) {
			IO.printLine();

			return;
		}
		if (!Commands.execute(command)) {
			IO.printLine(_("ERROR: Unknown command: %s"), command);

			return;
		}
		IO.printLine();
		commandLine.addItem(command); // update commands history
		commandLine.clear();
	}

	private void executeCommand() {
		executeCommand(commandLine.getText());
	}

	// public classes

	/** A registered console commands. */
	public static final class Commands extends Hashtable<String, ConsoleCommand> {

		// private

		private static Commands _instance;

		// public

		public static boolean execute(final String command) {
			ConsoleCommand action = getInstance().get(command);

			if (action == null)
				return false;

			action.onCommand();

			return true;
		}

		public static Commands getInstance() {
			if (_instance == null)
				_instance = new Commands();

			return _instance;
		}

		/**
		* Registers the console command.
		* Registered command is listed when you type "help" command.
		* @param name A command name (e.g. "foo")
		* @param action A console action
		*/
		public static void register(final String name, final ConsoleCommand action) {
			getInstance().put(name, action);
		}

		public static void show() {
			final Commands commands = getInstance();
			// sort items by command name
			ArrayList<String> sortedArray = TK.sortByKey(commands, new StringComparator());
			// print sorted items
			String description;
			for (String i : sortedArray) {
				description = commands.get(i).description.get();
				if (description != null)
					IO.printLine("%-20s%s", i, description);
			}
		}

		// private

		private Commands() { }

	}

	public static final class IO {

		// private

		private static boolean canLog;
		private static final int BUF_SIZE = 64 * 1024; // 64 KB
		private static final int MAX_BUF_SIZE = 512 * 1024; // 0,5 MB
		private static PrintStream oldErr;
		private static PrintStream oldOut;
		private static PrintStream printStream; // new print stream
		private static StringBuilder buf = new StringBuilder(BUF_SIZE);

		// public

		public synchronized static void flushLog() {
			if (!canLog || (buf == null))
				return;

			BufferedWriter logFile = null;
			try {
				File f = new File(FS.makeConfigPath("console.log"));
				try {
					// limit log file size
					if (f.length() > MAX_BUF_SIZE)
						f.delete();
				}
				catch (Exception exception) { }
				logFile = new BufferedWriter(new FileWriter(f.getPath(), true));
				logFile.write(buf.toString()); // write buf
			}
			catch (IOException exception) {
				// be quiet here
			}
			finally {
				FS.close(logFile);
			}
			buf = new StringBuilder(BUF_SIZE); // reset buf
		}

		/**
		* Prints a text to the @b stderr and to the console text area.
		* @param text A text to print
		*/
		public synchronized static void print(final String text) {
			if (oldErr != null)
				oldErr.print(text);
			if (canLog) {
				buf.append(text);
				if (Console._instance != null) {
					Console._instance.list.append(text);
					Console._instance.list.scrollToEnd();
				}
				if (buf.length() > BUF_SIZE)
					flushLog();
			}
		}

		/**
		* Prints a new line of text to the @b stderr and to the console text area.
		* @param text A text to print
		*/
		public synchronized static void printLine(final String text) {
			print(text + '\n');
		}

		/**
		* Prints a new line to the @b stderr and to the console text area.
		*/
		public synchronized static void printLine() {
			printLine("");
		}

		/**
		* Prints a new line of text to the @b stderr and to the console text area.
		* @param text A text to print
		* @param args Text arguments
		*/
		public synchronized static void printLine(final String text, final Object... args) {
			printLine(String.format(text, args));
		}

		public synchronized static void redirectOutput() {
			if (printStream != null)
				return;

			canLog = !Args.isSet("no-log");
			printStream = new PrintStream(new ConsoleStream());
			buf = new StringBuilder(BUF_SIZE);
			oldErr = err;
			oldOut = out;
			System.setErr(printStream);
			System.setOut(printStream);
		}

		/**
		* Restores @b stderr and @b stdout streams,
		* and flushes the console log buffer to disk.
		*/
		public synchronized static void shutDown() {
			if (oldErr != null)
				System.setErr(oldErr);

			if (oldOut != null)
				System.setOut(oldOut);

			flushLog();

			oldErr = null;
			oldOut = null;
			printStream = null;
			buf = null;
		}

		// private

		private IO() { }

	}

	// private classes

	private static final class ConsoleStream extends FilterOutputStream {

		// public

		ConsoleStream() {
			super(new ByteArrayOutputStream());
		}

		@Override
		public void write(final byte[] b) throws IOException {
			IO.print(new String(b));
		}

		@Override
		public void write(final byte[] b, final int off, final int len) throws IOException {
			IO.print(new String(b, off, len));
		}

	}

}
