// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.editors;

import static org.makagiga.common.UI._;

import java.awt.Color;
import java.text.MessageFormat;
import java.util.HashSet;
import java.util.Hashtable;
import javax.swing.JComponent;

import org.makagiga.MainWindow;
import org.makagiga.MetaInfo;
import org.makagiga.common.*;
import org.makagiga.plugins.PluginInfo;

/** The editor plugin base class. */
public abstract class Editor extends MPanel implements ReadOnly {

	// private

	private static Hashtable<String, Integer> ref = new Hashtable<String, Integer>();

	// public

	public final BooleanProperty alwaysSave = new BooleanProperty();
	public final BooleanProperty modified = new BooleanProperty();

	/**
	 * The active editor, or @c null if no editor.
	 * The active editor is managed by @ref Tabs.
	 * @b WARNING: This method is used internally by the @b Makagiga, and should not be called by the plugin.
	 */
	public static final Property<Editor> currentEditor = new Property<Editor>();

	/**
	 * The file meta info associated with this editor.
	 * @b WARNING: This method is used internally by the @b Makagiga, and should not be called by the plugin.
	 */
	public final Property<MetaInfo> metaInfo = new Property<MetaInfo>();

	public final Property<PluginInfo> info = new Property<PluginInfo>();

	// private

	private boolean readOnly;
	private String oldStatusText = "";

	// public API (override this!)

	/**
	 * Constructs a new editor.
	 * <pre>
	 * Example:
	 * ...
	 * public MyEditor() {
	 *   super("mycompany/myplugintype/myeditorname");
	 *   // init editor core
	 *   core = new MyEditorCore();
	 *   addCenter(core);
	 * }
	 * ...
	 * </pre>
	 */
	public Editor() {
		// increase reference count
		String key = getClass().getName();
		Integer value = ref.get(key);
		ref.put(key, (value == null) ? 1 : (value + 1));
	}

	public final void destroy() {
		onClose();
		// decrease reference count
		String key = getClass().getName();
		Integer value = ref.get(key);
		if (value == 1) {
			ref.remove(key);
			onLastClose();
		}
		else {
			ref.put(key, value - 1);
		}
	}

	/**
	 * Returns the editor core component.
	 * The component may not be @c null.
	 *
	 * EXAMPLE:
	 * @code
	 * public JComponent getCore() { return core; }
	 * @endcode
	 */
	public abstract JComponent getCore();

	public synchronized void setFocus() {
		getCore().requestFocusInWindow();
	}

	public synchronized final void showError(final Exception exception, final String message) {
		String s = MessageFormat.format(
			message + "\n\n" + _("Plugin said:") + "\n{1}",
			metaInfo.get().getPath(),
// !!!: "null" exception.getLocalizedMessage()
			exception.getLocalizedMessage()
		);
		MMessage.error(null, s);
	}

	public boolean isReadOnly() { return readOnly; }

	public void setReadOnly(final boolean value) { readOnly = value; }

	/**
	 * dummy
	 */
	public void loadFromFile() throws Exception { }

	public final void saveKeywords(final String value) {
		if (value == null) {
			metaInfo.get().setKeywords(null);

			return;
		}

		String[] keywords = value.replaceAll("\n", " ").split(" ");
		int length = keywords.length;

		if (length == 0) {
			metaInfo.get().setKeywords(null);

			return;
		}

		if (length == 1) {
			metaInfo.get().setKeywords(keywords[0]);

			return;
		}

		// remove duplicates
		HashSet<String> set = new HashSet<String>(length);
		for (String i : keywords) {
			// ignore words with length less than 3 chars
			if (i.length() > 2)
				set.add(i);
		}
		metaInfo.get().setKeywords(TK.join(set, " "));
	}

	/**
	 * Saves editor to file.
	 */
	public void saveToFile() throws Exception { }

	public void setMainColor(final Color value) { }

	/**
	 * OPTIONAL. Override this method to update editor actions.
	 * By default this function does nothing.
	 * <pre>
	 * Example:
	 * ...
	 * deleteAction.setEnabled(core.getSelectedRow() != -1);
	 * selectAllAction.setEnabled(!core.isEmpty());
	 * ...
	 * </pre>
	 * @see AppAction
	 */
	public synchronized void updateActions() { }

	// final public API

	public final Config getConfig() {
		return metaInfo.get().getConfig();
	}

	/**
	 * Returns a config key used to read/write the global plugin configuration.
	 * @see getGlobalConfig
	 */
	public final String getConfigEntry(final String configKey) {
		return info.get().id + "." + configKey;
	}

	public synchronized static final JComponent getCurrentCore() {
		Editor editor = currentEditor.get();

		if (editor == null)
			return null;

		return editor.getCore();
	}

	public synchronized final String getStatusText() { return oldStatusText; }

	public synchronized final void setStatusText(final String value) {
		oldStatusText = value;
		if (currentEditor.get() == this)
			MainWindow.getInstance().setEditorStatusText(value);
	}

	public synchronized final void setStatusText(final String value, final Object... args) {
		setStatusText(MessageFormat.format(value, args));
	}

	/**
	 * Displays a message box with information that editor is locked (read only).
	 * @return @c true if editor is locked
	 */
	public synchronized final boolean showLockInfo() {
		if (!readOnly)
			return false;

		MMessage.info(
			null,
			_("File \"{0}\" is locked (read only).\nClick \"Unlock\" if you want to edit this file."),
			metaInfo
		);

		return true;
	}

	/**
	 * This function calls @ref org.makagiga.editors.EditorConfig.saveConfig,
	 * and writes configuration file to the disk.
	 * @see EditorConfig
	 */
	public final void syncConfig() {
		if (this instanceof EditorConfig)
			EditorConfig.class.cast(this).saveConfig(Config.getGlobal(), getConfig());
		Config.getGlobal().saveToFile();
	}

	public synchronized void updateStatusText() { }

	public synchronized void onClose() { }

	public synchronized void onLastClose() { }

}
