// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.editors;

import javax.swing.Icon;

import org.makagiga.common.*;
import org.makagiga.plugins.Pluggable;
import org.makagiga.plugins.PluginException;
import org.makagiga.plugins.PluginInfo;
import org.makagiga.plugins.PluginManager;

/** dummy */
public abstract class EditorPlugin implements Pluggable {

	// public

	/**
	 * An array of file types which can be exported from the Makagiga.
	 * Quick info:
	 * - PROPERTIES KEY: String.x.exportTypes
	 * - REQUIRED: No
	 * - EXAMPLE: .html .txt
	 *
	 * Files types description (also works with import types; use ):
	 * Quick info:
	 * - PROPERTIES KEY: String.x.exportTypes.TYPE.description
	 * - REQUIRED: Yes
	 * - EXAMPLE: String.x.exportTypes.txt.description=A text file
	 * - NOTE: Use "importTypes" for import types
	 */
	public final Property<FileType[]> exportTypes = new Property<FileType[]>();

	/**
	 * An array of file types handled by this plugin.
	 * Quick info:
	 * - PROPERTIES KEY: String.x.fileTypes
	 * - REQUIRED: Yes
	 * - EXAMPLE: .txt
	 * - EXAMPLE 2: .txt .foo
	 */
	public final Property<FileType[]> fileTypes = new Property<FileType[]>();

	/**
	 * An array of file types which can be imported to the Makagiga.
	 * Quick info:
	 * - PROPERTIES KEY: String.x.importTypes
	 * - REQUIRED: No
	 * - EXAMPLE: .html .txt
	 */
	public final Property<FileType[]> importTypes = new Property<FileType[]>();

	public static final String TYPE = "editor";

	// public

	/**
	 * Constructs a new editor.
	 */
	public abstract Editor createEditor();

	/**
	 * Creates a new editor instance for the specified @p path.
	 * @return A new editor instance if successful; otherwise @c null
	 */
	public static Editor createEditorForPath(final String path) throws Exception {
		MLogger.info("PLUGINS: Creating editor for file name \"%s\"", path);

		String type = FS.getType(path);
		if (type == null) {
			MLogger.error("PLUGINS: No type for file name \"%s\"", path);

			return null;
		}

		PluginInfo info = findPluginForType(type);
		if (info == null) {
			MLogger.error("PLUGINS: Editor not found for file name \"%s\"", path);

			return null;
		}

		// create editor
		Editor editor = fromInfo(info).createEditor();
		// assign plugin info with editor
		editor.info.set(info);

		return editor;
	}

	public static Icon findIconForType(final String type) {
		PluginInfo info = findPluginForType(type);

		return (info == null) ? null : info.getIcon();
	}

	public static PluginInfo findPluginForType(final String type) {
		for (PluginInfo i : PluginManager.getInfos(TYPE)) {
			if (fromInfo(i).hasType(type))
				return i;
		}

		return null;
	}

	public static EditorPlugin fromInfo(final PluginInfo info) {
		return EditorPlugin.class.cast(info.plugin.get());
	}

	/**
	 * Returns the default file type.
	 * The default file type is used in "New File" function.
	 * @see org.makagiga.NewFileDialog
	 */
	public FileType getDefaultType() {
		if (TK.isEmpty(fileTypes.get()))
			return null;

		return fileTypes.get()[0];
	}

	/**
	 * Returns @c true if plugin supports file of @p type.
	 * @p type is a file name extension (e.g. ".png").
	 */
	public boolean hasType(final String type) {
		FileType[] types = fileTypes.get();

		if (TK.isEmpty(types))
			return false;

		for (FileType i : types) {
			if (i.is(type))
				return true;
		}

		return false;
	}

	public void onDestroy() throws PluginException { }

	public void onInit(final Config config, final PluginInfo info) throws PluginException {
		FileType[] types = parseTypes(config, "x.fileTypes");

		if (types == null)
			throw new PluginException("No \"String.x.fileTypes\" specified");

		fileTypes.set(types);
		exportTypes.set(parseTypes(config, "x.exportTypes"));
		importTypes.set(parseTypes(config, "x.importTypes"));
	}

	// private

	private FileType[] parseTypes(final Config config, final String key) {
		String value = config.read(key);

		if (TK.isEmpty(value))
			return null;

		String[] suffixes = value.split(" ");

		if (TK.isEmpty(suffixes))
			return null;

		FileType[] types = new FileType[suffixes.length];
		String description;
		for (int i = 0; i < suffixes.length; i++) {
			description = config.read(key + ".description" + suffixes[i]);
			types[i] = new FileType(suffixes[i], description, false);
		}

		return types;
	}

	// public classes

	/** A file type. */
	public static final class FileType {

		// private

		private boolean internal;
		private String description;
		private String type;

		// public

		/**
		 * Constructs a new file type.
		 * @param type A file name extension
		 * @param description A description
		 * @param internal If @c true, the file type is handled internally by Makagiga
		 * @see toString Use this function to get type name
		 */
		public FileType(final String type, final String description, final boolean internal) {
			this.internal = internal;
			this.description = description;
			this.type = type;
		}

		/**
		 * Returns description.
		 */
		public String getDescription() { return description; }

		/**
		 * Returns @c true if @p value equals this file type.
		 * @b NOTE: String case is ignored.
		 */
		public boolean is(final String value) {
			if (type == null)
				return false;

			if (value == null)
				return false;

			return type.equalsIgnoreCase(value);
		}

		/**
		 * Returns @c true if file type is handled internally by Makagiga.
		 */
		public boolean isInternal() { return internal; }

		/**
		 * Returns type name.
		 */
		@Override
		public String toString() { return type; }

	}

}
