// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.editors.image;

import static java.awt.Color.*;
import static java.awt.event.KeyEvent.*;

import static org.makagiga.common.UI._;

import java.awt.Color;
import java.awt.Point;
import javax.swing.JComponent;

import org.makagiga.common.*;
import org.makagiga.editors.*;
import org.makagiga.plugins.PluginMenu;

/** The image editor. */
public final class ImageEditor extends Editor
	implements
		EditorConfig,
		EditorDesigner,
		EditorExport,
		PluginMenu
{

	// private

	private ImageEditorCore core;

	// shared
	private static UndoAction undoAction;
	private static ClearAction clearAction;
	private static ShowGridAction showGridAction;

	// public

	/**
	 * Constructs a new image editor.
	 */
	public ImageEditor() {
		core = new ImageEditorCore(this);
		addCenter(new MScrollPane(core));

		// init share actions
		if (undoAction == null)
			undoAction = new UndoAction();
		if (clearAction == null)
			clearAction = new ClearAction();
		if (showGridAction == null)
			showGridAction = new ShowGridAction();
	}

	/**
	 * @see org.makagiga.editors.Editor.getCore
	 */
	public JComponent getCore() { return core; }

	/**
	 * Returns @c true if grid is visible.
	 */
	public static boolean getShowGrid() { return showGridAction.isSelected(); }

	@Override
	public boolean isReadOnly() {
		return core.readOnly.get();
	}

	@Override
	public void setReadOnly(final boolean value) {
		super.setReadOnly(value);
		core.readOnly.set(value);
		core.updateCursor();
		getEditorDesigner().setReadOnly(value);
	}

	@Override
	public void loadFromFile() throws Exception {
		core.loadFromFile(metaInfo.get().getPath());
		setSize(core.getCanvas().getWidth(), core.getCanvas().getHeight());
	}

	@Override
	public synchronized void onLastClose() {
		undoAction = null;
		clearAction = null;
		showGridAction = null;
	}

	@Override
	public void saveToFile() throws Exception {
		core.saveToFile(metaInfo.get().getPath());
		saveKeywords(core.getCanvas().getWidth() + " " + core.getCanvas().getHeight());
	}

	@Override
	public void setMainColor(final Color value) {
		if (value == null)
			core.setBackground(WHITE);
		else
			core.setBackground(value);
	}

	@Override
	public synchronized void updateActions() {
		undoAction.setEnabled(core.canUndo());
		clearAction.setEnabled(!isReadOnly());
	}

	@Override
	public synchronized void updateStatusText() {
		String text = "";
		Point p = core.getMousePosition();
		if (p != null) {
			text = String.format("X = %d, Y = %d", p.x, p.y);
			if (p.x > core.getCanvas().getWidth() || p.y > core.getCanvas().getHeight())
				text = " [!] " + text;
		}
		if (modified.get())
			text += " [" + _("Modified") + "]";
		setStatusText(text);
	}

	// public editor interfaces

	// EditorConfig

	public void loadConfig(final Config global, final Config local) {
		core.getBrush().color.set(global.readColor(getConfigEntry("brush.color"), BLUE));
		core.getBrush().setSize(
			global.readInt(
				getConfigEntry("brush.size"),
				Brush.DEFAULT_SIZE,
				Brush.MIN_SIZE,
				Brush.MAX_SIZE
			)
		);
		showGridAction.setSelected(global.read(getConfigEntry("view.grid"), true));
	}

	public void saveConfig(final Config global, final Config local) {
		global.write(getConfigEntry("brush.color"), core.getBrush().color);
		global.write(getConfigEntry("brush.size"), core.getBrush().getSize());
		global.write(getConfigEntry("view.grid"), showGridAction.isSelected());
	}

	// EditorDesigner

	public Designer getEditorDesigner() { return ImageEditorDesigner.getInstance(); }

	public void updateEditorDesigner() {
		MLogger.XXX("ImageEditor.updateEditorDesigner()");
		ImageEditorDesigner designer = (ImageEditorDesigner)getEditorDesigner();
		designer.setBrush(core, core.getBrush());
	}

	// EditorExport

	public void beginExport() {
		core.setExportMode(true);
	}

	public void endExport() {
		core.setExportMode(false);
	}

	public void exportFile(final EditorPlugin.FileType type, final String path) throws Exception {
		if (type.is(".bmp"))
			core.saveToFileAs("bmp", path);
// MUSTANG: GIF read/write support
/*
		else if (type.is(".gif"))
			core.saveToFileAs("gif", path) ? ExportResult.OK : ExportResult.ERROR;
*/
		else if (type.is(".jpeg"))
			core.saveToFileAs("jpg", path);
		else if (type.is(".png"))
			core.saveToFileAs("png", path);
// !!!: throw exception if type is not supported
	}

	// PluginMenu

	public void updateMenu(final String type, final MMenu menu) {
		if (type.equals(EDIT_MENU)) {
			menu.add(undoAction);
			menu.addSeparator();
			menu.add(clearAction);
		}
		else if (type.equals(VIEW_MENU)) {
			menu.addCheckBox(showGridAction);
		}
	}

	public void updateToolBar(final String type, final MToolBar toolBar) {
		if (type.equals(EDITOR_TOOL_BAR)) {
			toolBar.add(undoAction);
		}
	}

	// private classes

	private static final class ClearAction extends EditorAction<ImageEditor, ImageEditorCore> {

		// public

		ClearAction() {
			super(_("C&lear"), "ui/clear", VK_U, CTRL_MASK);
			setHTMLHelp(_("Clears all the image."));
		}

		public void onAction(final ImageEditor editor, final ImageEditorCore core) {
			if (MMessage.confirmWithIcon(
				null,
				"ui/clear",
				_("Delete image?")
			))
				core.clear();
		}

	}

	private static final class ShowGridAction extends AppToggleAction {

		// public

		public ShowGridAction() {
			super(_("Show &Grid"), "grid");
			setHTMLHelp(_("Check this option if you want to see the grid under images."));
		}

		public void onToggle(final boolean isSelected) {
			ImageEditor editor = (ImageEditor)currentEditor.get();
			if (editor != null) {
				editor.core.repaint();
				editor.syncConfig();
			}
		}

	}

	private static final class UndoAction extends EditorAction<ImageEditor, ImageEditorCore> {

		UndoAction() {
			super(_("&Undo"), "ui/undo", VK_Z, CTRL_MASK);
			setHTMLHelp(_("Undoes the last operation."));
		}

		public void onAction(final ImageEditor editor, final ImageEditorCore core) {
			core.undo();
		}

	}

}
