// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.editors.text;

import static org.makagiga.common.UI._;

import java.awt.Color;
import java.awt.Toolkit;
import java.awt.datatransfer.FlavorEvent;
import java.awt.datatransfer.FlavorListener;
import java.io.BufferedReader;
import javax.swing.JComponent;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.StyleSheet;

import org.makagiga.common.*;
import org.makagiga.editors.Editor;
import org.makagiga.editors.EditorExport;
import org.makagiga.editors.EditorPlugin;
import org.makagiga.editors.EditorSearch;
import org.makagiga.plugins.PluginMenu;

// TODO: 2.0: autoconvert "http://" to hyperlinks
public final class TextEditor extends Editor
	implements
		DocumentListener,
		EditorExport,
		EditorSearch,
		FlavorListener,
		PluginMenu
{

	// private

	private TextEditorCore core;

	// public

	public TextEditor() {
		core = new TextEditorCore();
		addCenter(core);
		// detect clipboard changes and update "paste" action
		Toolkit.getDefaultToolkit().getSystemClipboard().addFlavorListener(this);
	}

	public void changedUpdate(final DocumentEvent e) {
// !!!: "changedUpdate" fired after LAF change
		setModified();
	}

	public void flavorsChanged(final FlavorEvent e) {
// !!!: do not try to paste a non-text data (disable "pasteAction")
	}

	/**
	 * @see org.makagiga.editors.Editor.getCore
	 */
	public JComponent getCore() { return core; }

	public void insertUpdate(final DocumentEvent e) {
		setModified();
	}

	@Override
	public boolean isReadOnly() {
		return !core.isEditable();
	}

	@Override
	public void setReadOnly(final boolean value) {
		super.setReadOnly(value);
		core.setEditable(!value);
	}

// !!!: warn if .html file is very large [0.9.3]
	@Override
	public void loadFromFile() throws Exception {
		core.setContentType(MTextPane.TEXT_HTML_UTF_8);
		core.setDocument(new HTMLDocument());
		core.setEditorKit(new HTMLEditorKit());
		// add document change event handler
		core.getDoc().addDocumentListener(this);

		HTMLEditorKit kit = core.getKit();
		StyleSheet css = kit.getStyleSheet();
		css.addRule("body { font-family: \"SansSerif\"; }");

		BufferedReader reader = null;
		try {
			reader = FS.getUTF8Reader(metaInfo.get().getPath());
			kit.read(reader, core.getDoc(), 0);
		}
// FIXME: 2.0: ChangedCharSetException
		finally {
			FS.close(reader);
		}
		MText.installUndoManager(core); // do this after file load
		core.setCaretPosition(0);
// !!!: TextEditor designer
/*
		core.setSelectionStart(0);
		core.setSelectionEnd(10);
		MutableAttributeSet attr = new SimpleAttributeSet(core.getCharacterAttributes());
		StyleConstants.setBackground(attr, YELLOW);
		core.setCharacterAttributes(attr, true);
*/
	}

	@Override
	public synchronized void onClose() {
		Toolkit.getDefaultToolkit().getSystemClipboard().removeFlavorListener(this);
	}

	public void removeUpdate(final DocumentEvent e) {
		setModified();
	}

	@Override
	public void saveToFile() throws Exception {
		core.saveToFileAsHTML(metaInfo.get().getPath());
		saveKeywords(core.getPlainText());
	}

	@Override
	public void setMainColor(final Color value) {
		core.setColors(value);
	}

	@Override
	public synchronized void updateStatusText() {
		setStatusText(modified.get() ? _("Modified") : null);
	}

	// public editor interfaces

	// EditorExport

	public void beginExport() { }

	public void endExport() { }

	public void exportFile(final EditorPlugin.FileType type, final String path) throws Exception {
		if (type.is(".html"))
			core.saveToFileAsHTML(path);
		else if (type.is(".txt"))
			core.saveToFileAsText(path);
// !!!: throw exception if type is not supported
	}

	// EditorSearch

	public EditorSearch.Result findText(final String text) {
		String allText = core.getPlainText();

		// no text?
		if (TK.isEmpty(allText))
			return Result.EMPTY;

		// case insensitive
		allText = allText.toUpperCase();
		String textToFind = text.toUpperCase();

		// search from the current cursor position
		int foundAt = allText.indexOf(textToFind, core.getCaretPosition());

		// found?
		if (foundAt != -1) {
			// yes, move cursor and select the found text
			core.setCaretPosition(foundAt);
			core.setSelectionStart(foundAt);
			core.setSelectionEnd(foundAt + textToFind.length());

			return Result.FOUND;
		}

// !!!: wrap search
		return Result.NOT_FOUND;
	}

	public EditorSearch.Result findNextText(final String text) {
		return findText(text);
	}

	public String getDefaultSearchText() {
		return core.getSelectedText();
	}

	// PluginMenu

	public void updateMenu(final String type, final MMenu menu) {
		if (type.equals(EDIT_MENU))
			menu.addItemsFromMenu(MText.createMenu(core));
	}

	public void updateToolBar(final String type, final MToolBar toolBar) {
		if (type.equals(EDITOR_TOOL_BAR)) {
			toolBar.add(MText.getAction(core, MText.UNDO));
			toolBar.add(MText.getAction(core, MText.REDO));
			toolBar.addSeparator();
			toolBar.add(MText.getAction(core, MText.CUT));
			toolBar.add(MText.getAction(core, MText.COPY), MToolBar.SHOW_TEXT);
			toolBar.add(MText.getAction(core, MText.PASTE));
		}
	}

	// private

	private void setModified() {
		modified.yes();
		updateStatusText();
	}

}
