// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.editors.todo;

import static java.awt.Color.*;
import static java.awt.event.KeyEvent.*;

import static org.makagiga.common.UI._;

import java.awt.Color;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.print.PrinterException;
import javax.swing.JComponent;
import javax.swing.event.DocumentEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.JTableHeader;

import org.makagiga.common.*;
import org.makagiga.editors.*;
import org.makagiga.plugins.PluginMenu;

/** The @b Todo editor. */
public final class TodoEditor extends Editor
	implements
		EditorConfig,
		EditorExport,
		EditorPrint,
		EditorSearch,
		PluginMenu
{

	// private

	private TodoEditorCore core;

	// shared
	private static ClearAction clearAction;
	private static DeleteAction deleteAction;
	private static DeleteCompletedAction deleteCompletedAction;
	private static PasteLinesAction pasteLinesAction;
	private static SelectAllAction selectAllAction;

	private MButton addItemButton;
	private MPanel newTodoPanel;
	private MTextField newTodoText;

	// public

	/**
	 * Constructs a @b Todo editor.
	 */
	public TodoEditor() {
// !!!: detect changes and save only if file was actually modified
		alwaysSave.yes();

		// table
		core = new TodoEditorCore(this);
		addCenter(core);

		MPanel topPanel = MPanel.createBorderPanel();

		// new todo input
		MLabel newTodoLabel = new MLabel(_("&Add a new item:"));
		newTodoPanel = MPanel.createHBoxPanel();
// !!!: MTable and the tab key (Shift+Tab keys should navigate to MTextField)
		newTodoText = new MTextField() {
			@Override
			protected void onChange(final DocumentEvent e) {
				addItemButton.setEnabled(!isEmpty());
			}
		};
		newTodoText.setEnterPressAction(new SimpleAction() {
			public void onAction() {
				addItemButton.doClick();
			}
		} );
		newTodoLabel.setLabelFor(newTodoText);
		newTodoPanel.addGap();
		newTodoPanel.add(newTodoLabel);
		newTodoPanel.addGap();
		newTodoPanel.add(newTodoText);
		newTodoPanel.addGap();
		addItemButton = new MButton(FS.getIcon("ui/ok", 16)) {
			@Override
			protected void onClick() {
				core.addSummary(newTodoText.getText()); // add item
				core.selectRow(core.getRowCount() - 1); // select new item
				newTodoText.clear();
			}
		};
		addItemButton.setEnabled(false);
		addItemButton.setToolTipText(UI.makeHTML(_("Add item to the list.")));
		newTodoPanel.add(addItemButton);
		topPanel.addNorth(newTodoPanel);

		// header
// TODO: 2.0: MTableHeader
// TODO: 2.0: tool tips
		JTableHeader header = core.getTableHeader();
		header.setReorderingAllowed(false);
		topPanel.addCenter(header);
		addNorth(topPanel);

		// init shared actions
		if (clearAction == null)
			clearAction = new ClearAction();
		if (deleteAction == null)
			deleteAction = new DeleteAction();
		if (deleteCompletedAction == null)
			deleteCompletedAction = new DeleteCompletedAction();
		if (pasteLinesAction == null)
			pasteLinesAction = new PasteLinesAction();
		if (selectAllAction == null)
			selectAllAction = new SelectAllAction();

		// add selection event handler
		core.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(final ListSelectionEvent e) {
				updateActions();
				updateStatusText();
			}
		} );
	}

	/**
	 * @see org.makagiga.editors.Editor.getCore
	 */
	public JComponent getCore() { return core; }

	/**
	 * @see org.makagiga.editors.Editor.isReadOnly
	 */
	@Override
	public boolean isReadOnly() {
		return !newTodoText.isEnabled();
	}

	/**
	 * @see org.makagiga.editors.Editor.setReadOnly
	 * @param value @ref org.makagiga.editors.Editor.setReadOnly
	 */
	@Override
	public void setReadOnly(final boolean value) {
		super.setReadOnly(value);
		newTodoText.setEnabled(!value);
		newTodoPanel.setVisible(!value);
	}

	/**
	 * @see org.makagiga.editors.Editor.loadFromFile
	 */
	@Override
	public void loadFromFile() throws Exception {
		core.loadFromFile(metaInfo.get().getPath());
	}

	/**
	 * @see org.makagiga.editors.Editor.onLastClose
	 */
	@Override
	public synchronized void onLastClose() {
		clearAction = null;
		deleteAction = null;
		deleteCompletedAction = null;
		pasteLinesAction = null;
		selectAllAction = null;
	}

	/**
	 * @see org.makagiga.editors.Editor.saveToFile
	 */
	@Override
	public void saveToFile() throws Exception {
		core.saveToFileAsXML(metaInfo.get().getPath());
		saveKeywords(core.getKeywords());
	}

	/**
	 * @see org.makagiga.editors.Editor.setMainColor
	 * @param value @ref org.makagiga.editors.Editor.setMainColor
	 */
	@Override
	public void setMainColor(final Color value) {
		if (value == null)
			core.setBackground(WHITE);
		else
			core.setBackground(value);
	}

	/**
	 * @see org.makagiga.editors.Editor.updateActions
	 */
	@Override
	public synchronized void updateActions() {
		clearAction.setEnabled(!core.isEmpty() && !isReadOnly());
		deleteAction.setEnabled((core.getSelectedRow() != -1) && !isReadOnly());
		deleteCompletedAction.setEnabled(!core.isEmpty() && !isReadOnly());
		pasteLinesAction.setEnabled(!isReadOnly());
		selectAllAction.setEnabled(!core.isEmpty());
	}

	/**
	 * @see org.makagiga.editors.Editor.updateStatusText
	 */
	@Override
	public synchronized void updateStatusText() {
		int c = core.getRowCount();
		int s = core.getSelectedRowCount();
		if (s == 0) {
			setStatusText((c == 0) ? _("No items") : _("No items selected"));
		}
		else {
			if (s == c)
				setStatusText(_("Selected items: {0} (All)"), c);
			else
				setStatusText(_("Selected items: {0}"), s);
		}
	}

	// public editor interfaces

	// EditorConfig

	public void loadConfig(final Config global, final Config local) {
		core.selectRow(local.readInt(getConfigEntry("currentRow"), 0));
	}

	public void saveConfig(final Config global, final Config local) {
		local.write(getConfigEntry("currentRow"), core.getSelectedRow());
	}

	// EditorExport

	public void beginExport() {
		core.exportMode = true;
	}

	public void endExport() {
		core.exportMode = false;
	}

	public void exportFile(final EditorPlugin.FileType type, final String path) throws Exception {
		if (type.is(".html"))
			core.saveToFileAsHTML(path, false);
		else if (type.is(".mgtodo") || type.is(".xml"))
			core.saveToFileAsXML(path);
		else if (type.is(".txt"))
			core.saveToFileAsText(path);
		else if (type.is(".xhtml"))
			core.saveToFileAsHTML(path, true);
// !!!: throw exception if type is not supported
	}

	// EditorPrint

	public boolean printDocument() throws PrinterException {
// TODO: 2.0: i18n in print dialog
		return core.print();
	}

	// EditorSearch

	public EditorSearch.Result findText(final String text) {
		return core.searchFor(text, 0);
	}

	public EditorSearch.Result findNextText(final String text) {
		return core.searchFor(text, 1);
	}

	public String getDefaultSearchText() {
		return core.getSelectedSummary();
	}

	// PluginMenu

	public void updateMenu(final String type, final MMenu menu) {
		if (type.equals(EDIT_MENU)) {
			menu.add(pasteLinesAction);
			menu.addSeparator();
			menu.add(deleteCompletedAction);
			menu.addSeparator();
			menu.add(deleteAction);
			menu.add(clearAction);
			menu.addSeparator();
			menu.add(selectAllAction);
		}
	}

	public void updateToolBar(final String type, final MToolBar toolBar) {
		if (type.equals(EDITOR_TOOL_BAR)) {
			toolBar.add(deleteCompletedAction, MToolBar.SHOW_TEXT);
			toolBar.add(deleteAction, MToolBar.SHOW_TEXT);
		}
	}

	// private classes

	private static final class ClearAction extends EditorAction<TodoEditor, TodoEditorCore> {

		// public

		ClearAction() {
			super(_("C&lear"), "ui/clear", VK_U, CTRL_MASK);
			setHTMLHelp(_("Deletes all items."));
		}

		public void onAction(final TodoEditor editor, final TodoEditorCore core) {
			if (MMessage.confirmWithIcon(
				null,
				"ui/clear",
				_("Delete all items?")
			))
				core.deleteAll();
		}

	}

	private static final class DeleteAction extends EditorAction<TodoEditor, TodoEditorCore> {

		// public

		DeleteAction() {
			super(_("&Delete Selected"), "ui/delete", VK_DELETE);
			setHTMLHelp(_("Deletes all the selected items."));
		}

		public void onAction(final TodoEditor editor, final TodoEditorCore core) {
			if (MMessage.confirmWithIcon(
				null,
				"ui/delete",
				_("Delete all SELECTED items?\n\nSelected items: {0}"),
				core.getSelectedRowCount()
			))
				core.deleteSelected();
		}

	}

	private static final class DeleteCompletedAction extends EditorAction<TodoEditor, TodoEditorCore> {

		// public

		DeleteCompletedAction() {
			super(_("Delete &Completed"), "ui/ok", VK_DELETE, SHIFT_MASK);
			setHTMLHelp(_("Deletes all the completed items."));
		}

		public void onAction(final TodoEditor editor, final TodoEditorCore core) {
			int completedItems = core.getCompletedItemCount();
			if (completedItems == 0) {
				MStatusBar.setWarning(_("No completed items."));

				return;
			}

			if (MMessage.confirmWithIcon(
				null,
				"ui/ok",
				_("Delete all COMPLETED items?\n\nCompleted items: {0}"),
				completedItems
			))
				core.deleteCompletedItems();
		}

	}

	private static final class PasteLinesAction extends EditorAction<TodoEditor, TodoEditorCore> {

		// public

		public PasteLinesAction() {
			super(_("&Paste (1 line of text = 1 item)"), "ui/paste", VK_V, SHIFT_MASK + CTRL_MASK);
			setHTMLHelp(_("Pastes from the clipboard.<br>Each line of text is pasted as a new item."));
		}

		public void onAction(final TodoEditor editor, final TodoEditorCore core) {
// !!!: clipboard wrapper
			Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
			String data;
			try {
				data = (String)clipboard.getData(DataFlavor.stringFlavor);
			}
			catch (Exception exception) {
				MLogger.exception(exception);
				MStatusBar.setError(_("Paste failed: {0}", exception.getLocalizedMessage()));

				return;
			}
			if (!TK.isEmpty(data)) {//!!!refactor
				String[] lines = data.split("\n");
				for (String i : lines)
					core.addSummary(i);
			}
		}

	}

	private static final class SelectAllAction extends EditorAction<TodoEditor, TodoEditorCore> {

		// public

		SelectAllAction() {
			super(_("Select &All"), null, VK_A, CTRL_MASK);
			setHTMLHelp(_("Selects all items."));
		}

		public void onAction(final TodoEditor editor, final TodoEditorCore core) {
			core.selectAll();
		}

	}

}
