// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.editors.todo;

import java.awt.Color;
import java.io.BufferedInputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Date;
import javax.swing.DefaultCellEditor;
import javax.swing.table.TableColumn;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import org.makagiga.common.*;
import org.makagiga.editors.Editor;
import org.makagiga.editors.EditorSearch;

// !!!: context menu
// TODO: 2.0: column hidding
// TODO: 2.0: task summary at bottom
/** A todo editor core. */
public final class TodoEditorCore extends MTable {

	// protected

	protected boolean exportMode; /**< @c true if editor is in export mode. */

	// private

	private Editor editor;

	// xml tag names
	private static final String MG_ITEM = "item";
	private static final String MG_SUMMARY = "summary";
	private static final String MG_PRIORITY = "priority";
	private static final String MG_COMPLETE = "complete";
	private static final String MG_DATE_TIME = "datetime";
	private static final String MG_VALUE = "value";

	// public

	/**
	 * Constructs a todo editor core.
	 * @param editor A parent editor
	 */
	public TodoEditorCore(final Editor editor) {
		this.editor = editor;
		for (Column i : Column.values()) {
			model.addColumn(i.toString());
		}

// !!!: remember/restore columns width
		for (Column i : Column.values())
			getTodoColumn(i).setPreferredWidth(i.getPreferredWidth());

// FIXME: 2.0: column/window resize deletes uncommited value (JAVA BUG #4330950)
		// setup editors and renderers
		DefaultCellEditor e;
		// done column
		e = new DefaultCellEditor(new DoneComponent());
		getTodoColumn(Column.DONE).setCellEditor(e);
		getTodoColumn(Column.DONE).setCellRenderer(new DoneComponent());
		// summary column
// !!!: "summary" and "date/time" column conflicts with menus' VK_DELETE
		getTodoColumn(Column.SUMMARY).setCellEditor(new SummaryEditor());
		getTodoColumn(Column.SUMMARY).setCellRenderer(new SummaryRenderer());
		// priority column
		e = new DefaultCellEditor(new PriorityEditor());
		getTodoColumn(Column.PRIORITY).setCellEditor(e);
		getTodoColumn(Column.PRIORITY).setCellRenderer(new PriorityRenderer());
		// complete column
		e = new DefaultCellEditor(new CompleteEditor());
		getTodoColumn(Column.COMPLETE).setCellEditor(e);
		getTodoColumn(Column.COMPLETE).setCellRenderer(new CompleteRenderer());
		// date/time column
// !!!: 4-digit year format
		getTodoColumn(Column.DATE_TIME).setCellEditor(new DateTimeEditor());
		getTodoColumn(Column.DATE_TIME).setCellRenderer(new DateTimeRenderer());
	}

	/**
	 * Adds a new summary to the todo list.
	 * @param text A summary text
	 */
	public void addSummary(final String text) {
		addTodo(text, Priority.DEFAULT, 0, 0);
	}

	/**
	 * Adds a new item to the todo list.
	 * @param summary A summary text
	 * @param priority A priority
	 * @param complete A complete status (0-100)
	 * @param dateTime A date/time value
	 */
	public void addTodo(final String summary, final Priority priority, final int complete, final long dateTime) {
		Object[] rows = {
			complete == 100,
			summary,
			priority,
			complete,//!!!validate
			new Date(dateTime)
		};
		model.addRow(rows);
	}

	/**
	 * Deletes all the completed items.
	 */
	public void deleteCompletedItems() {
		doneEdit();
		int selectedRow = getSelectedRow();
		try {
			for (int row = getRowCount() - 1; row >= 0; --row) {
				if (isCompleted(row))
					model.removeRow(row);
			}
		}
		catch (ArrayIndexOutOfBoundsException exception) {
			MLogger.exception(exception);
		}
		selectRow(selectedRow);
	}

	/**
	 * Returns the @p column class.
	 */
	@Override
	public Class<?> getColumnClass(final int column) {
		if (column == Column.DONE.ordinal())
			return Boolean.class;

		return super.getColumnClass(column);
	}

	/**
	 * Returns a number of the completed items.
	 */
	public int getCompletedItemCount() {
		int retVal = 0;
		int rowCount = getRowCount();
// !!!: iterators
		for (int row = 0; row < rowCount; row++)
			if (isCompleted(row))
				retVal++;

		return retVal;
	}

	/**
	 * Returns a list of all words as single string.
	 */
	public String getKeywords() {
		int rowCount = getRowCount();

		if (rowCount == 0)
			return null;

		String summary;
		StringBuilder result = new StringBuilder(1024);
		for (int row = 0; row < rowCount; row++) {
			summary = model.getValueAt(row, Column.SUMMARY.ordinal()).toString();
			if (!TK.isEmpty(summary)) {
				result.append((row == rowCount - 1) ? summary : (summary + " "));
			}
		}

		return result.toString();
	}

	/**
	 * Returns the summary of the selected item, or @c null if no selection.
	 */
	public String getSelectedSummary() {
		try {
			return model.getValueAt(getSelectedRow(), Column.SUMMARY.ordinal()).toString();
		}
		catch (ArrayIndexOutOfBoundsException exception) {
			return null;
		}
	}

	/**
	 * Returns the selection background color.
	 */
	@Override
	public Color getSelectionBackground() {
		return exportMode ? getBackground() : super.getSelectionBackground();
	}

	/**
	 * Returns @c true if cell at @p row and @p column is editable (not read only).
	 */
	@Override
	public boolean isCellEditable(final int row, final int column) {
		return
			!editor.isReadOnly() &&
			(
				(row == getSelectedRow()) || // workaround for combo box editor
				(column == Column.DONE.ordinal())
			);
	}

	public boolean isCompleted(final int row) {
		return getValueAt(row, Column.DONE.ordinal()).equals(Boolean.TRUE);
	}

	/**
	 * Loads items from @p path.
	 */
	public void loadFromFile(final String path) throws Exception {
		File file = new File(path);

		if (file.length() == 0)
			return; // new file, ok

		XMLReader xmlReader = XMLReaderFactory.createXMLReader();
		xmlReader.setContentHandler(new TodoHandler());
		xmlReader.parse(new InputSource(new BufferedInputStream(new FileInputStream(file))));
	}

	/**
	 * Saves all items as HTML file.
	 * @param path A target file name
	 * @param xhtml If @c true file will be saved as XHTML Strict; otherwise as HTML Transitional
	 */
	public void saveToFileAsHTML(final String path, final boolean xhtml) throws Exception {
		HTMLBuilder builder = new HTMLBuilder(xhtml ? HTMLBuilder.XHTML_STRICT : HTMLBuilder.HTML_TRANSITIONAL);
		builder.beginHTML(editor.metaInfo.toString(), HTMLBuilder.UTF_8);
		builder.addMeta("Generator", AppInfo.getTitle());
		builder.beginStyle();
		builder.beginRule("body");
			builder.addAttr("background-color", getBackground());
			builder.addAttr("font-family", "sans-serif");
		builder.endRule();
		builder.beginRule("table");
			builder.addAttr("border", "0px solid black");
			builder.addAttr("border-spacing", "0px");
			builder.addAttr("width", "100%");
		builder.endRule();
// TODO: 2.0: WYSIWYG
		builder.beginRule("td");
			builder.addAttr("border", "1px solid black");
			builder.addAttr("padding", "2px");
		builder.endRule();
		builder.endStyle();
		builder.beginDoc();
		builder.beginTag("table");
		builder.beginTag("tr");
// !!!: text alignment in header
		for (Column i : Column.values()) {
			builder.beginTag(
				"td",
				"style", "text-align: center"
			);
			builder.appendChars("b", i.toString());
			builder.endTag("td");
		}
		builder.endTag("tr");
		int rowCount = getRowCount();
		for (int row = 0; row < rowCount; row++)
			writeItemAsHTML(builder, row);
		builder.endTag("table");
		builder.endDoc();
		builder.saveToFile(path);
	}

	/**
	 * Saves all items as plain text file.
	 * @param path A target file name
	 */
	public void saveToFileAsText(final String path) throws Exception {
		BufferedWriter writer = null;
		try {
			writer = FS.getUTF8Writer(path);
			for (Column i : Column.values()) {
				writer.write(i.toString());
				if (i == Column.LAST)
					writer.newLine();
				else
					writer.write("  ");
			}
			writer.newLine();
			int rowCount = getRowCount();
			for (int row = 0; row < rowCount; row++)
				writeItemAsText(writer, row);
		}
		finally {
			FS.close(writer);
		}
	}

	/**
	 * Saves all items as XML file.
	 * @param path A target file name
	 */
	public void saveToFileAsXML(final String path) throws Exception {
		XMLBuilder builder = new XMLBuilder();
		builder.beginTag(
			"todo",
			"version", "1.0"
		);
		int rowCount = getRowCount();
		for (int row = 0; row < rowCount; row++)
			writeItemAsXML(builder, row);
		builder.endTag("todo");
		builder.saveToFile(path);
	}

	/**
	 * Searches for the specified text.
	 * @param text A text to find
	 * @param delta A value which will be added to the selected row index. Selected row index + @p delta is a start index.
	 */
	public EditorSearch.Result searchFor(final String text, final int delta) {
		int rowCount = getRowCount();

		if (rowCount == 0)
			return EditorSearch.Result.EMPTY;

		int startIndex = getSelectedRow() + delta;
		// wrap search
// !!!: wrap search [0.9.3]
		if (startIndex > rowCount - 1)
			startIndex = 0;

		String summary;
		for (int row = startIndex; row < rowCount; row++) {
			summary = model.getValueAt(row, Column.SUMMARY.ordinal()).toString();
			if (TK.containsIgnoreCase(summary, text)) {
				selectRow(row);

				return EditorSearch.Result.FOUND;
			}
		}

		return EditorSearch.Result.NOT_FOUND;
	}

	/**
	 * Sets cell value.
	 * @param value A new cell value
	 * @param row A row
	 * @param column A column
	 */
	@Override
	public void setValueAt(final Object value, final int row, final int column) {
		// "done" column
		if (column == Column.DONE.ordinal()) {
			boolean b = (Boolean)value;
			super.setValueAt(b ? 100 : 0, row, Column.COMPLETE.ordinal());
		}
		// "complete" column
		else if (column == Column.COMPLETE.ordinal()) {
			int i = (Integer)value;
			super.setValueAt(i == 100, row, Column.DONE.ordinal());
		}
		super.setValueAt(value, row, column);
	}

	// private

	private TableColumn getTodoColumn(final Column index) {
		return getColumnModel().getColumn(index.ordinal());
	}

	private void writeItemAsHTML(final HTMLBuilder builder, final int row) {
		int complete = (Integer)model.getValueAt(row, Column.COMPLETE.ordinal());

		builder.beginTag("tr");

		// done
		builder.beginTag(
			"td",
			"style", "font-family: monospace; text-align: center"
		);
		builder.appendLine("[" + ((complete == 100) ? "X" : " ") + "]");
		builder.endTag("td");

		// summary
		builder.beginTag("td");
		builder.appendLine(TK.escapeXML(model.getValueAt(row, Column.SUMMARY.ordinal()).toString()), true);
		builder.endTag("td");

		// priority
		Priority p = (Priority)model.getValueAt(row, Column.PRIORITY.ordinal());
		Color pc = p.getColor();
		builder.beginTag(
			"td",
			"style", "background-color: " + ((pc == null) ? "inherit" : TK.toHTMLTriplet(p.getColor())) + "; text-align: center"
		);
		builder.appendLine(p.toString());
		builder.endTag("td");

		// complete
		builder.beginTag(
			"td",
			"style", "text-align: center"
		);
		builder.appendLine(Integer.toString(complete) + "%");
		builder.endTag("td");

		// date/time
		builder.beginTag("td");
		builder.appendLine(DateTimeRenderer.formatDate((Date)model.getValueAt(row, Column.DATE_TIME.ordinal())), true);
		builder.endTag("td");

		builder.endTag("tr");
	}

	private void writeItemAsText(final BufferedWriter writer, final int row) throws IOException {
		int complete = (Integer)model.getValueAt(row, Column.COMPLETE.ordinal());
		String s = String.format(
			"[%s]  %s\t%s\t%d%%\t%s",
			(complete == 100) ? "X" : " ", // "done" column
			model.getValueAt(row, Column.SUMMARY.ordinal()),
			model.getValueAt(row, Column.PRIORITY.ordinal()),
			complete,
			DateTimeRenderer.formatDate((Date)model.getValueAt(row, Column.DATE_TIME.ordinal()))
		);
		writer.write(s);
		writer.newLine();
	}

	private void writeItemAsXML(final XMLBuilder builder, final int row) {
		builder.beginTag(
			MG_ITEM,
			MG_PRIORITY, Integer.toString(Priority.class.cast(model.getValueAt(row, Column.PRIORITY.ordinal())).ordinal()),
			MG_COMPLETE, model.getValueAt(row, Column.COMPLETE.ordinal()).toString()
		);

		// summary
		builder.appendChars(
			MG_SUMMARY,
			model.getValueAt(row, Column.SUMMARY.ordinal()).toString()
		);

		// date/time
		try {
			Date date = (Date)model.getValueAt(row, Column.DATE_TIME.ordinal());
			builder.singleTag(
				MG_DATE_TIME,
				MG_VALUE, Long.toString(date.getTime())
			);
		}
		catch (ClassCastException exception) {
			builder.singleTag(
				MG_DATE_TIME,
				MG_VALUE, "0"
			);
		}

		builder.endTag(MG_ITEM);
	}

	// private classes

	private final class TodoHandler extends XMLHandler {

		// private

		private int complete;
		private long dateTime;
		private Priority priority;
		private String summary;

		// public

		@Override
		public void onEnd(final String name) throws SAXException {
			if (name.equals(MG_ITEM))
				addTodo(summary, priority, complete, dateTime);
			else if (name.equals(MG_SUMMARY))
				summary = getChars();
		}

		@Override
		public void onStart(final String name) throws SAXException {
			if (name.equals(MG_ITEM)) {
				complete = getIntegerValue(MG_COMPLETE, 0);
				priority = Priority.fromString(getStringValue(MG_PRIORITY));
				summary = "";
			}
			else if (name.equals(MG_DATE_TIME)) {
				dateTime = getLongValue(MG_VALUE, 0);
			}
		}

	}

}
