// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.plugins;

import static org.makagiga.common.UI._;

import java.util.Collection;
import javax.swing.BorderFactory;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;

import org.makagiga.common.*;

/** A plugin list. */
public final class PluginList extends MList<PluginInfo> {

	// public

	public static final int INCLUDE_DISABLED = 1 << 0; /**< Display disabled plugins. */
	public static final int PACKED_MODE = 1 << 1; /**< No empty line between name and description. */
	public static final int SHOW_COPYRIGHT = 1 << 2; /**< Show plugin copyright. */
	public static final int SHOW_INFO_IF_EMPTY = 1 << 3; /**< Display information message if no plugins of the specified type. */
	public static final int SHOW_VERSION = 1 << 4; /**< Show plugin version. */

	// public

	/**
	 * Constructs a new plugin list.
	 * @param type A plugin type (e.g. @c EditorPlugin.TYPE)
	 * @param flags Flags (e.g. @ref SHOW_VERSION)
	 */
	public PluginList(final String type, final int flags) {
		boolean includeDisabled = (flags & INCLUDE_DISABLED) != 0;
		boolean showInfoIfEmpty = (flags & SHOW_INFO_IF_EMPTY) != 0;
		setCellRenderer(new PluginListCellRenderer(flags));
		setSingleSelectionMode();

		Collection<PluginInfo> infos = PluginManager.getInfos(type, includeDisabled);
		if (infos != null) {
			for (PluginInfo i : infos)
				addItem(i);
		}

		if (isEmpty() && showInfoIfEmpty)
			MMessage.info(null, _("No plugin installed."));
	}

	/**
	 * Selects a plugin with @p id,
	 * or first item if plugin with @p id was not found in the list.
	 */
	public void selectByID(final String id) {
		if (isEmpty())
			return;

		MListIterator i = listIterator();
		while (i.hasNext()) {
			if (i.next().id.equals(id)) {
				setSelectedIndex(i.getIndex());

				return;
			}
		}
		setSelectedIndex(0);
	}

	// private classes

	private static final class PluginListCellRenderer extends SimpleListCellRenderer<PluginInfo> {

		// private

		private boolean packedMode;
		private boolean showCopyright;
		private boolean showVersion;
		private Border marginBorder;

		// public

		public PluginListCellRenderer(final int flags) {
			packedMode = (flags & PACKED_MODE) != 0;
			showCopyright = (flags & SHOW_COPYRIGHT) != 0;
			showVersion = (flags & SHOW_VERSION) != 0;
			marginBorder = BorderFactory.createEmptyBorder(0, 10 /* left margin */, 0, 0);
		}

		public void onRender(final PluginInfo value) {
			setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createBevelBorder(hasFocus ? BevelBorder.LOWERED : BevelBorder.RAISED),
				marginBorder
			));
			setIcon(value.getIcon());
			setIconTextGap(10); // right margin
			String copyright =
				showCopyright
				? "<span style=\"font-weight: normal\">" + value.copyright + "</span><br>"
				: "";
			String description = value.description.get();
			if (description == null)
				description = "[" + _("No description") + "]";
			String lineSeparator = packedMode ? "" : "<br>";
			String version =
				showVersion
				? "&nbsp;&nbsp;<span style=\"font-weight: normal\">" + value.version + "</span>"
				: "";
			String disabled =
				value.enabled.get()
				? ""
				: "<span style=\"color: red\">[" + _("Disabled") + "] </span>";
			setText(UI.makeHTML(
				disabled + "<span style=\"font-weight: bold\">" + value + "</span>" +
				version + "<br>" +
				copyright +
				lineSeparator +
				"<span style=\"font-weight: normal\">" + description + "</span>"
			));
		}

	}

}
