// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.settings;

import static org.makagiga.common.UI._;

import org.makagiga.common.*;

/** The "General" settings. */
public final class GeneralSettings extends SettingsPage implements SettingsAction {

	// private

	private boolean toolTipsDetailsOriginal;
	private MCheckBox autosave;
	private MCheckBox enabledSplashScreen;
	private MCheckBox minimizeOnClose;
	private MCheckBox toolTipsDetails;
	private MCheckBox toolTipsEnabled;

	// public

	/**
	 * Constructs a @b General settings.
	 */
	public GeneralSettings() {
		super(_("General"));
		setVBoxLayout();

		// splash screen
		enabledSplashScreen = new MCheckBox(_("&Splash Screen"));
		enabledSplashScreen.setToolTipText(UI.makeHTML(_("Check this option to enable the splash screen.")));
		add(enabledSplashScreen);

		// autosave
		autosave = new MCheckBox(_("Auto&save"));
		autosave.setToolTipText(UI.makeHTML(_("Check this option to save files automatically<br>every 10 minutes.")));
		add(autosave);

		// minimize on close
		minimizeOnClose = new MCheckBox(_("&Minimize Instead of Close"));
		minimizeOnClose.setToolTipText(UI.makeHTML(UI.makeHTML(_("Check this option to minimize the main window<br>after click on the <b>X</b> button."))));
		add(minimizeOnClose);

		// -
		addStretch();

		// tool tips
		MPanel toolTipsPanel = MPanel.createHBoxPanel();
		toolTipsPanel.setTitle(_("Tool Tips"));
		// enabled
		toolTipsEnabled = new MCheckBox(_("&Enabled")) {
			@Override
			public void onToggle(final boolean isSelected) {
				toolTipsDetails.setEnabled(isSelected);
			}
		};
		toolTipsEnabled.setToolTipText(UI.makeHTML(_("Check this option to enable tool tip texts.")));
		toolTipsPanel.add(toolTipsEnabled);
		// details
		toolTipsDetails = new MCheckBox(_("More &Tool Tips"));
		toolTipsDetails.setToolTipText(UI.makeHTML(_("Check this option to enable more tool tip texts.")));
		toolTipsPanel.add(toolTipsDetails);
		add(toolTipsPanel);
	}

	/**
	 * Returns @c true if autosave is enabled.
	 */
	public static boolean readAutosave() {
		return Config.getGlobal().read("session.autosave", true);
	}

	/**
	 * Returns @c true if minimizing on close is enabled.
	 */
	public static boolean readMinimizeOnClose() {
		return Config.getGlobal().read("window.minimizeOnClose", false);
	}

	/**
	 * Returns @c true if splash screen is enabled.
	 */
	public static boolean readSplashScreenEnabled() {
		return Config.getGlobal().read("splashScreen.enabled", true);
	}

	/**
	 * Returns @c true if tool tips details are enabled.
	 */
	public static boolean readToolTipsDetails() {
		return Config.getGlobal().read("toolTips.details", false);
	}

	/**
	 * Returns @c true if tool tips are enabled.
	 */
	public static boolean readToolTipsEnabled() {
		return Config.getGlobal().read("toolTips.enabled", true);
	}

	// SettingsAction

	public void onInit() {
		autosave.setSelected(readAutosave());
		enabledSplashScreen.setSelected(readSplashScreenEnabled());
		minimizeOnClose.setSelected(readMinimizeOnClose());
		toolTipsDetails.setSelected(UI.toolTipsDetails.get());
		toolTipsDetailsOriginal = toolTipsDetails.isSelected();
		toolTipsEnabled.setSelected(readToolTipsEnabled());
		toolTipsDetails.setEnabled(toolTipsEnabled.isSelected());
	}

	public void onOK() {
		Config config = Config.getGlobal();
		config.write("session.autosave", autosave.isSelected());
		config.write("splashScreen.enabled", enabledSplashScreen.isSelected());
		config.write("window.minimizeOnClose", minimizeOnClose.isSelected());
		config.write("toolTips.details", toolTipsDetails.isSelected());
		config.write("toolTips.enabled", toolTipsEnabled.isSelected());
		needRestart.set(toolTipsDetails.isSelected() != toolTipsDetailsOriginal);
	}

}
