// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.settings;

import static org.makagiga.common.UI._;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.plaf.BorderUIResource;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.MetalTheme;
import javax.swing.plaf.metal.OceanTheme;

import org.makagiga.MainWindow;
import org.makagiga.common.*;

// TODO: 2.0: add synth style support (skin)
// TODO: 2.0: themes with high contrast (skin)
/** The "Look And Feel" (LAF) settings dialog. */
public final class LookAndFeelSettings extends SettingsPage implements SettingsAction {

	// public

	public static final int DEFAULT_ICON_SIZE = 24; /**< The default icon size. */
	public static final String DEFAULT = "DEFAULT"; /**< The default LAF (no @c UIManager.setLookAndFeel). */
	public static final String JAVA = "JAVA"; /**< The @b Metal LAF with @b Ocean theme. */
	public static final String SYSTEM = "SYSTEM"; /**< The LAF returned by the @c UIManager.getSystemLookAndFeelClassName(). */

	// private

	private boolean customColorsOriginal;
	private boolean customIconsOriginal;
	private int iconSizeOriginal;
	private MCheckBox customColors;
	private MCheckBox customIcons;
	private MComboBox<Icon> iconSize;
	private MComboBox<UIManager.LookAndFeelInfo> installedLookAndFeels;
	private String lookAndFeelClassName;

	// public

	/**
	 * Construct a <b>Look And Feel</b> settings dialog.
	 */
	public LookAndFeelSettings() {
		super(_("Look & Feel"));

		// installed look & feels
		installedLookAndFeels = new MComboBox<UIManager.LookAndFeelInfo>();
		installedLookAndFeels.setRenderer(new SimpleListCellRenderer<UIManager.LookAndFeelInfo>() {
			public void onRender(final UIManager.LookAndFeelInfo value) {
				setText(value.getName());
			}
		} );
		installedLookAndFeels.addItem(new UIManager.LookAndFeelInfo(_("Default"), DEFAULT));
		installedLookAndFeels.addItem(new UIManager.LookAndFeelInfo("Java" + UI.TM, JAVA));
		installedLookAndFeels.addItem(new UIManager.LookAndFeelInfo(_("System"), SYSTEM));
		UIManager.LookAndFeelInfo[] lookAndFeelInfos = UIManager.getInstalledLookAndFeels();
		for (UIManager.LookAndFeelInfo i : lookAndFeelInfos) {
			if (!i.getName().equals("GTK+") || OS.isJavaMustang())
				installedLookAndFeels.addItem(i);
		}
		MPanel installedLookAndFeelsPanel = MPanel.createVBoxPanel();
		installedLookAndFeelsPanel.setTitle(_("Select a Look & Feel"));
		installedLookAndFeelsPanel.add(installedLookAndFeels);
		LookAndFeel currentLookAndFeel = UIManager.getLookAndFeel();
		String s = currentLookAndFeel.getDescription();
		if (currentLookAndFeel.isNativeLookAndFeel())
			s += " [" + _("Native") + "]";
		installedLookAndFeelsPanel.addGap();
		MLabel currentLookAndFeelLabel = new MLabel();
		currentLookAndFeelLabel.setMnemonicEnabled(false);
		currentLookAndFeelLabel.setText(_("Current Look & Feel: {0}", s));
		installedLookAndFeelsPanel.add(currentLookAndFeelLabel);
		addNorth(installedLookAndFeelsPanel);

		installedLookAndFeelsPanel.addGap();
		customColors = new MCheckBox(_("C&ustom Colors"));
		installedLookAndFeelsPanel.add(customColors);
		customIcons = new MCheckBox(_("Cu&stom Icons"));
		installedLookAndFeelsPanel.add(customIcons);

		// icon size
		MPanel iconSizePanel = MPanel.createTitledPanel(_("Icon Size"));
		iconSize = new MComboBox<Icon>();
		iconSize.setRenderer(new SimpleListCellRenderer<Icon>() {
			public void onRender(final Icon value) {
				setText("(" + value.getIconWidth() + "x" + value.getIconHeight() + ")");
			}
		} );
		iconSize.addItem(FS.getIcon("makagiga", FS.MIN_ICON_SIZE));
		iconSize.addItem(FS.getIcon("makagiga", 24));
		iconSize.addItem(FS.getIcon("makagiga", 32));
		iconSize.addItem(FS.getIcon("makagiga", FS.MAX_ICON_SIZE));
		iconSizePanel.addCenter(iconSize);
		addSouth(iconSizePanel);
	}

	/**
	 * Applies the specified LAF.
	 * @param name An abstract name of the LAF (e.g. @ref DEFAULT)
	 * @param updateConfig @c true if configuration file and UI tree should be updated
	 */
	public static void applyLAF(final String name, final boolean updateConfig) {
		String abstractClassName = name;
		if (abstractClassName == null)
			abstractClassName = readLAFClassName();

		if (abstractClassName.equals(DEFAULT)) {
			Config.getGlobal().write("LAF.className", abstractClassName);
		}
		else {
			MDialog.setDefaultLookAndFeelDecorated(false);
			MFrame.setDefaultLookAndFeelDecorated(false);

			String realClassName = abstractClassName;
			try {
				if (TK.isEmpty(realClassName) || realClassName.equals(JAVA)) {
					setDefaultLAF(new OceanTheme());
				}
				else if (realClassName.equals(SYSTEM)) {
					realClassName = UIManager.getSystemLookAndFeelClassName();
					if (realClassName.contains("MetalLookAndFeel"))
						setDefaultLAF(new OceanTheme());
					else
						UIManager.setLookAndFeel(realClassName);
				}
				else if (realClassName.contains("MetalLookAndFeel")) {
					setDefaultLAF(new DefaultMetalTheme());
				}
				else {
					UIManager.setLookAndFeel(realClassName);
				}
				if (updateConfig) {
					SwingUtilities.updateComponentTreeUI(MainWindow.getInstance());
					Config.getGlobal().write("LAF.className", abstractClassName);
				}
			}
			catch (Exception exception) {
				MLogger.error("Could not apply LAF: abstractClassName=\"%s\", realClassName=\"%s\"", abstractClassName, realClassName);
				MLogger.exception(exception);
				setDefaultLAF(new OceanTheme());
			}
		}

		// init i18n (override defaults)
		UIDefaults defaults = UIManager.getDefaults();
		defaults.put("OptionPane.cancelButtonText", _("Cancel"));
		defaults.put("OptionPane.okButtonText", _("OK"));

		boolean customColors = readCustomColors();
		if (customColors) {
			// init tool tip
			UIManager.put("ToolTip.background", new ColorUIResource(UI.TOOL_TIP_BACKGROUND_COLOR));
			UIManager.put("ToolTip.foreground", new ColorUIResource(UI.TOOL_TIP_FOREGROUND_COLOR));
			Border border = BorderUIResource.getBlackLineBorderUIResource();
			UIManager.put("ToolTip.border", border);
			UIManager.put("ToolTip.borderInactive", border);
			// init tree
			UIManager.put("Tree.hash", UI.getUIColor("Tree.background")); // no lines
		}
		// init tree
		if (UI.customIcons.get()) {
			UIManager.put("Tree.collapsedIcon", FS.getDefaultIcon("ui/plus"));
			UIManager.put("Tree.expandedIcon", FS.getDefaultIcon("ui/minus"));
			UIManager.put("Tree.leafIcon", null);
		}
	}

	/**
	 * Returns @c true if custom icons are enabled.
	 */
	public static boolean readCustomIcons() {
		return Config.getGlobal().read("LAF.customIcons", !OS.isMac() && !OS.isWindows());
	}

	/**
	 * Reads a default icon size.
	 */
	public static int readIconSize() {
		return Config.getGlobal().readInt("LAF.iconSize", DEFAULT_ICON_SIZE);
	}

	// SettingsAction

	public void onInit() {
		customColorsOriginal = readCustomColors();
		customColors.setSelected(customColorsOriginal);
		customIconsOriginal = UI.customIcons.get();
		customIcons.setSelected(customIconsOriginal);

		// icon size
		int size = FS.iconSize.get();
		if (size == FS.DEFAULT_ICON_SIZE) {
			iconSize.setSelectedIndex(2);
		}
		else {
			MComboBox<Icon>.MComboBoxIterator i = iconSize.comboBoxIterator();
			while (i.hasNext()) {
				if (i.next().getIconWidth() == size) {
					iconSize.setSelectedIndex(i.getIndex());

					break; // for
				}
			}
		}
		iconSizeOriginal = iconSize.getSelectedIndex();

		// look & feel
		lookAndFeelClassName = readLAFClassName();

		if (lookAndFeelClassName == null)
			return;

		MComboBox<UIManager.LookAndFeelInfo>.MComboBoxIterator i = installedLookAndFeels.comboBoxIterator();
		while (i.hasNext()) {
			if (i.next().getClassName().equals(lookAndFeelClassName)) {
				installedLookAndFeels.setSelectedIndex(i.getIndex());

				break; // for
			}
		}
	}

	public void onOK() {
		Config config = Config.getGlobal();

		config.write("LAF.customColors", customColors.isSelected());
		if (customColors.isSelected() != customColorsOriginal)
			needRestart.yes();
		config.write("LAF.customIcons", customIcons.isSelected());
		if (customIcons.isSelected() != customIconsOriginal)
			needRestart.yes();

		// icon size
		int size = iconSize.getSelectedItem().getIconWidth();
		FS.iconSize.set(size);
		config.write("LAF.iconSize", size);
		if (iconSize.getSelectedIndex() != iconSizeOriginal)
			needRestart.yes();

		// look & feel
		UIManager.LookAndFeelInfo info = installedLookAndFeels.getSelectedItem();
		if (!info.getClassName().equals(lookAndFeelClassName)) {
			applyLAF(info.getClassName(), true);
			needRestart.yes();
// TODO: 2.0: option: MFrame.setDefaultLookAndFeelDecorated(true);
		}
	}

	// private

	private static boolean readCustomColors() {
		return Config.getGlobal().read("LAF.customColors", !OS.isMac() && !OS.isWindows());
	}

	private static String readLAFClassName() {
		return Config.getGlobal().read("LAF.className", (OS.isMac() || OS.isWindows()) ? SYSTEM : DEFAULT);
	}

	private static void setDefaultLAF(final MetalTheme theme) {
		try {
			MetalLookAndFeel metal = new MetalLookAndFeel();
			metal.setCurrentTheme(theme);
			UIManager.setLookAndFeel(metal);
		}
		catch (UnsupportedLookAndFeelException exception) {
			MLogger.exception(exception);
		}
	}

}
