// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.tools;

import static org.makagiga.common.UI._;

import java.util.Vector;
import javax.swing.event.HyperlinkEvent;

import org.makagiga.MainWindow;
import org.makagiga.common.*;
import org.makagiga.common.sun.SwingWorker;

// TODO: 2.0: preload Browser classes
// TODO: 2.0: "refresh" button
// TODO: 2.0: refresh help automatically after main menu change
/**
 * An universal browser.
 * Update, help, info...
 */
public final class Browser extends MPanel {

	// private

	private static Browser _instance;
	private History history = new History();
	private MTextPane content;
	private SwingWorker worker;

	// actions
	private final BackAction backAction = new BackAction();
	private final StopAction stopAction = new StopAction();

	// public

	/**
	 * Returns the browser content component.
	 */
	public synchronized MTextPane getContent() { return content; }

	/**
	 * Constructs and returns the instance of the Browser.
	 */
	public synchronized static Browser getInstance() {
		if (_instance == null)
			_instance = new Browser();

		return _instance;
	}

	/**
	 * Navigates to the @p url.
	 */
	public synchronized void goTo(final String url) {
		goTo(url, true);
	}

	/**
	 * Sets content to @p value.
	 */
	public synchronized void setText(final String value) {
// !!!: remove old style sheet before set text
		// clear stream description
		content.setForceReload();
		content.setContentType(MTextPane.TEXT_HTML_UTF_8);
		content.setText(value);
	}

	// private

	private Browser() {
		// tool bar
		MToolBar toolBar = new MToolBar();
		toolBar.add(backAction, MToolBar.SHOW_TEXT);
		toolBar.addSeparator();
		toolBar.add(stopAction);
// TODO: 2.0: "forward" action
		addNorth(toolBar);

		// content
		content = new MTextPane() {
			@Override
			protected synchronized boolean onHyperlinkClick(final HyperlinkEvent e) {
				// makagiga protocol
				if (e.getURL() == null) {
					goTo(e.getDescription());

					return true;
				}

				return false;
			}
		};
		content.setEditable(false);
		content.setNoMargin();
		addCenter(content);
	}

	private synchronized void back() {
		if (history.back())
			goTo(history.getCurrentURL(), false);
	}

	private synchronized void goTo(final String url, final boolean updateHistory) {
		if (processInternalURL(url))
			return;

		MLogger.info("Browse: " + url);

		if (updateHistory)
			history.addURL(url);

		if (worker != null)
			worker.interrupt();

		worker = new SwingWorker() {
			public Object construct() {
				try {
					UI.setWaitCursor(content, true);
					MStatusBar.setText(_("Loading: {0}", url));
					content.setContentType(MTextPane.TEXT_HTML_UTF_8);
					content.setPage(url);
				}
				catch (Exception exception) {
					setText(
						UI.HTML_BEGIN +
						_("<h1>Error</h1>Could not load page <b>{0}</b><h2>Details</h2>", url) +
						exception +
						UI.HTML_END
					);
					MLogger.exception(exception);
				}
				finally {
					UI.setWaitCursor(content, false);
					backAction.setEnabled(history.canBack());
					stopAction.setEnabled(false);
					MStatusBar.ready();
				}

				return null;
			}
		};
		stopAction.setEnabled(true);
		worker.start();
	}

	private synchronized boolean processInternalURL(final String url) {
		if (TK.isEmpty(url))
			return true; // ?

		if (!url.startsWith("makagiga:")) // makagiga "protocol"
			return false;

		if (url.equals("makagiga:about"))
			MMainWindow.about();
		else if (url.equals("makagiga:backup"))
			MainWindow.getInstance().backup();
		else if (url.equals("makagiga:help"))
			MainWindow.getInstance().help();
		else if (url.equals("makagiga:update"))
			MainWindow.getInstance().update();

		return true;
	}

	// private classes

	private final class BackAction extends AppAction {

		// public

		public BackAction() {
			super(_("Back"), "back");
			setEnabled(false);
		}

		public void onAction() {
			back();
		}

	}

	private final class StopAction extends AppAction {

		// public

		public StopAction() {
			super(_("Stop"), "stop");
			setEnabled(false);
		}

		public void onAction() {
			setEnabled(false);
			worker.interrupt();
			worker = null;
		}

	}

	private static final class History extends Vector<String> {

		// private

		private int index = -1;

		// public

		public void addURL(final String value) {
			index++;
			try {
				if (index == size() - 1)
					add(value);
				else
					insertElementAt(value, index);
			}
			catch (ArrayIndexOutOfBoundsException exception) {
				MLogger.exception(exception);
			}
		}

		public boolean back() {
			if (!canBack())
				return false;

			index--;

			return true;
		}

		public boolean canBack() {
			return index > 0;
		}

		public String getCurrentURL() {
			try {
				return get(index);
			}
			catch (ArrayIndexOutOfBoundsException exception) {
				return null;
			}
		}

	}

}
