// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.tools;

import static java.awt.Color.*;

import static org.makagiga.common.UI._;

import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import javax.swing.JComponent;
import javax.swing.SwingUtilities;

import org.makagiga.MainWindow;
import org.makagiga.QuickStart;
import org.makagiga.Sidebar;
import org.makagiga.Tabs;
import org.makagiga.common.*;
import org.makagiga.editors.Editor;

/** The <b>Live Tutorial</b>. */
public final class LiveTutorial extends JComponent {

	// private

	private ArrayList<Tutorial> tutorials;
	private static LiveTutorial _instance;

	// public

	/**
	 * Paints the <b>Live Tutorial</b> over the main window.
	 * @param graphics A graphics
	 */
	@Override
	public void paint(final Graphics graphics) {
		Graphics2D g = (Graphics2D)graphics;
		for (Tutorial i : tutorials)
			i.draw(g);
// TODO: 2.0 animation, fade, ?
	}

	/**
	 * Toggles the <b>Live tutorial</b>.
	 */
	public static void toggle() {
		// deactivate
		if ((_instance != null) && _instance.isVisible()) {
			_instance.setVisible(false);
			MainWindow.getInstance().setGlassPane(new MLabel());
			_instance = null;
		}
		// activate
		else {
			if (_instance == null)
				_instance = new LiveTutorial();
			MainWindow.getInstance().setGlassPane(_instance);
			_instance.setVisible(true);
		}
	}

	// private

	private LiveTutorial() {
		tutorials = new ArrayList<Tutorial>();

		// menu bar
		tutorials.add(new Tutorial(YELLOW, _("Main Menu - Press F10 to activate")) {
			public Component getComponent() {
				return MainWindow.getInstance().getMMenuBar();
			}
		} );

		// sidebar
		tutorials.add(new Tutorial(new Color(0xc0c0ff), _("Sidebar\n\nF4 = Toggle Designer\nF7 = Search for files\nF9 = Toggle Sidebar")) {
			public Component getComponent() {
				return Sidebar.getInstance();
			}
		} );

		// editor tool bar
		tutorials.add(new Tutorial(new Color(0xffa858), _("Editor Tool Bar\nEach open file has its own tool bar")) {
			public Component getComponent() {
				Editor editor = Tabs.getInstance().getSelectedTab();

				if (editor == null)
					return null;

				return MainWindow.getInstance().getEditorToolBar();
			}
		} );

		// tabs
		tutorials.add(new Tutorial(new Color(0xc0ffc0), _("Tabs - All the open files are here")) {
			public Component getComponent() {
				return MainWindow.getInstance().isQuickStart() ? null : Tabs.getInstance();
			}
		} );

		// editor quick find
		tutorials.add(new Tutorial(new Color(0xffdca8), _("Search in the active editor")) {
			public Component getComponent() {
				Editor editor = Tabs.getInstance().getSelectedTab();

				if (editor == null)
					return null;

				return EditorQuickFind.getInstance();
			}
		} );

		// status bar
		tutorials.add(new Tutorial(new Color(0xffc0c0), _("Status Bar - Info and minor error messages")) {
			public Component getComponent() {
				return MStatusBar.getGlobal();
			}
		} );
	}

	// private classes

	private static abstract class Tutorial {

		// private

		private static final BasicStroke BORDER_STROKE = new BasicStroke(5);
		private Color color;
		private static final Font FONT = new Font("Dialog", Font.BOLD, 14);
		private static FontMetrics fontMetrics;
		private Rectangle2D textRect;
		private String text;
		private String[] lines;

		// public

		public Tutorial(final Color color, final String text) {
			this.color = color;
			this.text = text;
			lines = text.split("\n");
		}

		public void draw(final Graphics2D g) {
			Component component = getComponent();

			if ((component == null) || !component.isVisible())
				return;

			if (TK.isEmpty(text))
				return;

			Rectangle r = SwingUtilities.convertRectangle(
				component.getParent(),
				component.getBounds(),
				_instance
			);

			Composite oldAlpha = g.getComposite();
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.8f));

			// border
			g.setColor(color);
			g.setStroke(BORDER_STROKE);
			g.drawRect(r.x, r.y, r.width, r.height);

			// text background
			g.setFont(FONT);
			if (fontMetrics == null)
				fontMetrics = g.getFontMetrics(FONT);
			if (textRect == null)
				textRect = fontMetrics.getStringBounds(text, g);
			int x = r.x;
			int y = r.y - 2;
			int w = r.width;
			int textHeight = (int)textRect.getHeight();
			int h = textHeight * lines.length;
			g.fillRect(x, y, w + 5, h + 5);

			// text
			g.setComposite(oldAlpha);
			g.setColor(BLACK);
			y += textHeight;
			for (String line : lines) {
				g.drawString(line, x + 4, y);
				y += textHeight;
			}
		}

		public abstract Component getComponent();

	}

}
