// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.tools;

import static org.makagiga.common.UI._;

import java.awt.image.BufferedImage;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import javax.imageio.ImageIO;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import org.makagiga.MainWindow;
import org.makagiga.common.*;

// TODO: 2.0: index of menus
// !!!: icon cache
/**
 * Generates a HTML help from the main menu actions.
 *
 * EXAMPLE:
 * @code
 * ...
 * Widget2Help w2h = new Widget2Help();
 * w2h.setMenuBar(getMMenuBar());
 * w2h.setOutputDir(FS.makeConfigPath("help"));
 * w2h.setTitle(AppInfo.getTitle());
 * w2h.generate();
 * ...
 * @endcode
 */
public final class Widget2Help {

	// private

	private ArrayList<Shortcut> shortcuts;
	private boolean highlightRow;
	private HTMLBuilder builder;
	private int uniqueImage;
	private MMenuBar menuBar;
	private String outputDir;
	private String title = "";

	// colors
	private static final String MENU_BACKGROUND = "#418edc";
	private static final String MENU_FOREGROUND = "white";
	private static final String MENU_ITEM_BACKGROUND_1 = "#edf4f9";
	private static final String MENU_ITEM_BACKGROUND_2 = "white";

	// public

	public Widget2Help() {
		shortcuts = new ArrayList<Shortcut>();
		builder = new HTMLBuilder(HTMLBuilder.HTML_TRANSITIONAL);
	}

	public void addShortcut(final String description) {
		addShortcut(description, 0, 0);
	}

	public void addShortcut(final String description, final int keyCode) {
		addShortcut(description, keyCode, 0);
	}

	public void addShortcut(final String description, final int keyCode, final int modifiers) {
		Shortcut shortcut = new Shortcut();
		shortcut.description.set(description);
		shortcut.keyCode.set(keyCode);
		shortcut.modifiers.set(modifiers);
		shortcuts.add(shortcut);
	}

	/**
	 * Starts the HTML generator.
	 */
	public boolean generate() {
		try {
			builder.beginHTML(title, HTMLBuilder.UTF_8);
			builder.addComment(AppInfo.fileInfo.get());
			builder.beginStyle();
			builder.setDefaultStyle();
			builder.beginRule("body");
				builder.addAttr("margin", "0px");
			builder.endRule();
			builder.beginRule("table");
				builder.addAttr("border", "0px solid black");
				builder.addAttr("border-spacing", "0px");
// TODO: 2.0: help header and index
				builder.addAttr("margin", "0px");
				builder.addAttr("width", "100%");
			builder.endRule();
			builder.endStyle();
			builder.beginDoc();
			builder.addAnchor("top");
			builder.beginTag("table");
			processMenuBar();
			processShortcuts();
			addScrollToTop();
			builder.endTag("table");
			builder.endDoc();
			builder.saveToFile(FS.makePath(outputDir, "index.html"));

			return true;
		}
		catch (IOException exception) {
			MLogger.exception(exception);

			return false;
		}
	}

	/**
	 * Sets menu bar to @p value.
	 */
	public void setMenuBar(final MMenuBar value) { menuBar = value; }

	/**
	 * Sets output directory to @p value.
	 * The directory and its subdirectories will
	 * be automatically created.
	 */
	public void setOutputDir(final String value) { outputDir = value; }

	/**
	 * Sets the help title to @p value.
	 */
	public void setTitle(final String value) { title = value; }

	// private

	private void addScrollToTop() {
		builder.beginTag("tr");
		builder.beginTag(
			"td",
			"colspan", "3",
			"style", getMenuItemStyle() + "; text-align: right"
		);
		builder.appendLine(null, true);
		builder.appendLine(builder.createLink("#top", _("Scroll to Top")));
		builder.endTag("td");
		builder.endTag("tr");

		highlightRow = !highlightRow;
	}

	private String getIconTag(final AppAction action) throws IOException {
		if (action == null)
			return null;

		return getIconTag(action.getIcon());
	}

	private String getIconTag(final Icon icon) throws IOException {
		if (icon == null)
			return null;

		int w = icon.getIconWidth();
		int h = icon.getIconHeight();

		if (w < 1 || h < 1)
			return null;

		BufferedImage bi = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
		UI.copyImage(ImageIcon.class.cast(icon).getImage(), bi);

		String iconName = "img_" + (uniqueImage++) + ".png";
		String iconPath = FS.makePath(outputDir, iconName);

		if (!ImageIO.write(bi, "png", new File(iconPath)))
			return null;

		return builder.createImage(null, iconName, "", null, w, h);
	}

	private String keyStrokeToText(final KeyStroke keyStroke) {
		String kc = KeyEvent.getKeyText(keyStroke.getKeyCode());
		String km = KeyEvent.getKeyModifiersText(keyStroke.getModifiers());
		String retVal = "";

		if (!TK.isEmpty(km))
			retVal += km;

		if (!TK.isEmpty(kc)) {
			if (TK.isEmpty(km))
				retVal += kc;
			else
				retVal += "+" + kc;
		}

		return retVal;
	}

	private void processMenu(final MMenu menu) throws IOException {
		if (menu == null)
			return;

		int l = menu.getItemCount();

		if (l == 0)
			return;

		String icon = getIconTag(menu.getComponentIcon());
		builder.beginTag(
			"tr",
			"style", getMenuStyle()
		);
		// menu icon
		builder.beginTag(
			"td",
			"style", "text-align: center"
		);
		builder.appendLine(icon, true);
		builder.endTag("td");
		// menu text
		builder.beginTag(
			"td",
			"colspan", "2"
		);
		builder.appendLine("<b><font size=\"+1\">" + menu.getText() + "</font></b>");
		builder.endTag("td");
		builder.endTag("tr");
		highlightRow = false;

		JMenuItem item;
		for (int i = 0; i < l; i++) {
			item = menu.getItem(i);
			if (item != null) {
				if (item instanceof MMenu) {
					if (
						(item != MainWindow.getInstance().getExportMenu()) &&
						(item != MainWindow.getInstance().getImportMenu())
					)
						processMenu((MMenu)item);
				}
				else if (item instanceof MMenuItem) {
					processMenuItem((MMenuItem)item);
				}
			}
		}

		addScrollToTop();
	}

	private void processMenuBar() throws IOException {
		if (menuBar == null)
			return;

		int l = menuBar.getMenuCount();

		if (l == 0)
			return;

		MMenu menu;
		for (int i = 0; i < l; i++) {
			menu = menuBar.getMenuAt(i);
			if (menu != MainWindow.getInstance().getBookmarksMenu())
				processMenu(menu);
		}
	}

	private void processMenuItem(final MMenuItem item) throws IOException {
		if (item == null)
			return;
		
		String text = item.getText();

		if (TK.isEmpty(text))
			return;

		AppAction action = item.getAction();
		KeyStroke keyStroke = null;
		String description = null;
		String icon = null;
		if (action != null) {
			description = action.getHTMLHelp();
			if (description != null) {
				description = description.replace(UI.HTML_BEGIN, "");
				description = description.replace(UI.HTML_END, "");
			}
			keyStroke = action.getAcceleratorKey();
			icon = getIconTag(action);
		}
		builder.beginTag(
			"tr",
			"style", getMenuItemStyle()
		);

		// icon
		builder.beginTag(
			"td",
			"style", "text-align: center"
		);
		builder.appendLine(icon, true);
		builder.endTag("td");
		// text
		builder.beginTag("td");
		String keyStrokeInfo = (keyStroke == null) ? "" : ("<br><br>" + keyStrokeToText(keyStroke));
		builder.appendLine("<b>" + text + "</b>" + keyStrokeInfo);
		builder.endTag("td");
		// description
		builder.beginTag("td");
		builder.appendLine(description, true);
		builder.endTag("td");

		builder.endTag("tr");

		highlightRow = !highlightRow;
	}

	public void processShortcuts() {
		if (shortcuts.isEmpty())
			return;

		for (Shortcut i : shortcuts) {
			// title
			if ((i.keyCode.get() == 0) && (i.modifiers.get() == 0)) {
				highlightRow = false;
				builder.beginTag(
					"tr",
					"style", getMenuStyle()
				);
				builder.beginTag(
					"td",
					"colspan", "3"
				);
				builder.appendLine("<b><font size=\"+1\">" + i.description + "</font></b>");
				builder.endTag("td");
				builder.endTag("tr");
			}
			else {
				builder.beginTag("tr");
				builder.beginTag(
					"td",
					"colspan", "2",
					"style", getMenuItemStyle()
				);
				int keyCode = i.keyCode.get();
				int modifiers = i.modifiers.get();
				KeyStroke keyStroke = KeyStroke.getKeyStroke(keyCode, modifiers);
				builder.appendLine(TK.escapeXML(keyStrokeToText(keyStroke)));
				builder.endTag("td");
				builder.beginTag(
					"td",
					"style", getMenuItemStyle()
				);
				builder.appendLine(TK.escapeXML(i.description.get()));
				builder.endTag("td");
				builder.endTag("tr");

				highlightRow = !highlightRow;
			}
		}
	}

	// private

	private String getMenuItemStyle() {
		return "background-color: " + (highlightRow ? MENU_ITEM_BACKGROUND_1 : MENU_ITEM_BACKGROUND_2);
	}

	private String getMenuStyle() {
		return String.format("background-color: %s; color: %s", MENU_BACKGROUND, MENU_FOREGROUND);
	}

	// private classes

	private static final class Shortcut {

		// public

		public final IntegerProperty keyCode = new IntegerProperty();
		public final IntegerProperty modifiers = new IntegerProperty();
		public final StringProperty description = new StringProperty();

	}

}
