// Copyright 2005 Konrad Twardowski
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.makagiga.tree;

import static java.awt.event.KeyEvent.*;

import static org.makagiga.common.UI._;

import java.awt.Color;
import java.io.File;
import java.io.IOException;
import javax.swing.JToolTip;
import javax.swing.event.MenuEvent;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

import org.makagiga.MainWindow;
import org.makagiga.MetaInfo;
import org.makagiga.NewFileDialog;
import org.makagiga.PropertiesDialog;
import org.makagiga.Tabs;
import org.makagiga.common.*;
import org.makagiga.editors.Editor;
import org.makagiga.editors.EditorPlugin;

// TODO: 2.0: virtual folders
// !!!: Shift+F10 = context menu
// TODO: 2.0: mount external directories as virtual directory
/** The @b Tree tab. */
public final class Tree extends MPanel {

	// private

	private JToolTip infoBar;//!!!
	private MMenu labelMenu;
	private String lastNewFolderName;
	private static Tree _instance;
	private TreeList treeList;

	// private actions
	private final ColorAction colorAction = new ColorAction();
	private final DeleteItemAction deleteItemAction = new DeleteItemAction();
	private final LabelAction labelAction = new LabelAction();
	private final NewFileAction newFileAction = new NewFileAction();
	private final NewFolderAction newFolderAction = new NewFolderAction();
	private final PropertiesAction propertiesAction = new PropertiesAction();
	private final RenameAction renameAction = new RenameAction();
	private final ToggleBookmarkAction toggleBookmarkAction = new ToggleBookmarkAction();

	// public

	/**
	 * Changes color of the selected file or folder.
	 */
	public void color() {
		MetaInfo metaInfo = treeList.getSelectedMetaInfo();

		Color color = metaInfo.getColor();
		MColorChooser chooser = new MColorChooser(MainWindow.getInstance(), _("File Color"), color, color) {
			@Override
			protected void onDefault() {
				setColor(null);
				done(MDialogAction.Code.ACCEPTED);
			}
		};

		if (!chooser.isOK())
			return;

		color = chooser.getColor();

		metaInfo.setColor(color);
		Tabs.getInstance().setTabInfo(metaInfo);
		treeList.repaint();
	}

	/**
	 * Constructs and returns the @b Tree menu.
	 * @param parent A parent menu or @c null
	 * @param fileActions If @c true only file specific actions will be included
	 */
	public MMenu createTreeMenu(final MMenu parent, final boolean fileActions) {
// TODO: 2.0: "Open", "Open with..."
		MMenu treeMenu = (parent == null) ? new MMenu() : parent;
		if (!fileActions) {
			treeMenu.add(newFileAction);
			treeMenu.add(newFolderAction);
		}
		treeMenu.add(renameAction);
		treeMenu.add(toggleBookmarkAction);
		treeMenu.addSeparator();
		treeMenu.add(deleteItemAction);
		treeMenu.addSeparator();
		treeMenu.add(colorAction);
		if (labelMenu == null) {
			labelMenu = new MMenu(_("&Label"));
			labelMenu.addMenuListener(new MMenuAdapter() {
				@Override
				public void menuSelected(final MenuEvent e) {
					MMenu m = (MMenu)e.getSource();
					if (m.isEmpty())
						createLabelMenu(m);
				}
			} );
		}
		labelMenu.setEnabled(labelAction.isEnabled());
		treeMenu.add(labelMenu);
		treeMenu.add(propertiesAction);

		return treeMenu;
	}

	/**
	 * Returns the @c colorAction.
	 */
	public ColorAction getColorAction() { return colorAction; }

	/**
	 * Constructs and returns a new instance of the @b Tree.
	 */
	public static Tree getInstance() {
		if (_instance == null)
			new Tree(); // set "_instance" in constructor

		return _instance;
	}

	/**
	 * Returns the @c renameAction.
	 */
	public RenameAction getRenameAction() { return renameAction; }

	/**
	 * Returns the selected folder item,
	 * or @c null if no selection.
	 */
	public TreeListItem getSelectedFolderItem() {
		TreeListItem item = treeList.getSelectedItem();

		if (item == null)
			return null;

		if (item.getData().isFile())
			return (TreeListItem)item.getParentItem();

		return item;
	}

	/**
	 * Updates actions.
	 * (e.g. disable @c renameAction if selected item is locked)
	 */
	public void updateActions() {
		boolean dataRoot = false;
		boolean isFile = false;
		boolean isFolder = false;
		boolean isLocked = false;
		boolean isSelection = false;
		MetaInfo metaInfo = null;
		TreeListItem item = treeList.getSelectedItem();
		isSelection = item != null;
		if (isSelection) {
			metaInfo = item.getData();
			isFile = metaInfo.isFile();
			isFolder = metaInfo.isFolder();
			isLocked = metaInfo.isLocked();
			dataRoot = metaInfo.isDataRoot();
		}
		boolean enabled = !isLocked && isSelection && !dataRoot;
		colorAction.setEnabled(enabled);
		deleteItemAction.setEnabled(!isLocked && (isFile || (isFolder && !item.hasItems())));
		labelAction.setEnabled(isSelection && !dataRoot);
		newFileAction.setEnabled(isSelection);
		newFolderAction.setEnabled(isSelection);
		propertiesAction.setEnabled(isSelection && !dataRoot);
		renameAction.setEnabled(enabled);

		// toggle bookmark action
		if ((metaInfo == null) || !isFile) {
			toggleBookmarkAction.setMode(ToggleBookmarkAction.Mode.NO_FILE, null);
		}
		else {
			toggleBookmarkAction.setMode(
				metaInfo.bookmark.get()
					? ToggleBookmarkAction.Mode.REMOVE
					: ToggleBookmarkAction.Mode.ADD,
				metaInfo
			);
		}
	}

	// private

	private Tree() {
		_instance = this;
		MPanel toolBarAndTreePanel = MPanel.createBorderPanel();
		// init tool bar
		MToolBar topToolBar = new MToolBar();
		topToolBar.add(newFileAction, MToolBar.SHOW_TEXT);
		topToolBar.add(newFolderAction);
		toolBarAndTreePanel.addNorth(topToolBar);

		// init tree list
		treeList = TreeList.getInstance();
		toolBarAndTreePanel.addCenter(treeList);

		// init info
		infoBar = new JToolTip();
		infoBar.setBorder(null);//!!!limit comment height
		infoBar.setBackground(java.awt.Color.WHITE);
		infoBar.setForeground(java.awt.Color.BLACK);

		/*MSplitPane splitPane = new MSplitPane(MSplitPane.VERTICAL_SPLIT);
		splitPane.setTopComponent(toolBarAndTreePanel);
		splitPane.setBottomComponent(new MScrollPane(infoBar));//!!!!!log
		addCenter(splitPane);*/
		
		addCenter(toolBarAndTreePanel);

		// add selection event handler
		treeList.addTreeSelectionListener(new TreeSelectionListener() {
			public void valueChanged(final TreeSelectionEvent e) {
				updateActions();

				// update info pane
				MetaInfo metaInfo = treeList.getSelectedMetaInfo();
				if (metaInfo == null)
					infoBar.setTipText(UI.makeHTML(_("Select an item to view its info")));
				else
					infoBar.setTipText(metaInfo.getToolTipText());
			}
		} );
		updateActions();
	}

// TODO: 2.0: custom labels
	private void createLabelMenu(final MMenu parent) {
		parent.add(new LabelItemAction(_("Star"), "labels/star"));
		parent.add(new LabelItemAction(_("Valuable"), "labels/valuable"));
		parent.add(new LabelItemAction(_("Important"), "labels/important"));
		parent.add(new LabelItemAction(_("Todo"), "labels/todo"));
		parent.add(new LabelItemAction(_("Unknown"), "labels/unknown"));
		// add actions to the menu
		MMenu emotionMenu = new MMenu(_("&Emotion"), "labels/emotion/1");
		parent.setHTMLHelp(_("Select a label for the selected file or folder."));
		for (int i = 1; i < 12; ++i)
			emotionMenu.add(new LabelItemAction("labels/emotion/" + i));
		parent.add(emotionMenu);
		parent.addSeparator();
		parent.add(new DefaultLabelAction());
// TODO: 2.0: a submenu with recent labels
	}

	private boolean pathExists(final String path) {
		if (FS.exists(path)) {
			MMessage.fileError(
				null,
				_("File already exists"),
				path
			);

			return true;
		}

		return false;
	}

	private void newFile() {
		TreeListItem parentItem = getSelectedFolderItem();

		if (parentItem == null)
			return;

		MetaInfo metaInfo = parentItem.getData();

		NewFileDialog newFileDialog = new NewFileDialog(MainWindow.getInstance());
		String name = newFileDialog.getName();

		// cancel?
		if (name == null)
			return;

		// check for "" before escape
		if (!TreeFS.isFileNameValid(name))
			return;

		name = TK.escapeURL(name);

		if (!TreeFS.isFileNameValid(name))
			return;

		String path = FS.makePath(metaInfo.getPath(), name);

		if (pathExists(path))
			return;

		File file = new File(path);
		try {
			if (file.createNewFile()) {
				// add item
				TreeListItem fileItem = TreeListItem.createFileItem(
					path,
					newFileDialog.getInfo()
				);
				metaInfo = fileItem.getData();
				metaInfo.setCreatedTime();
				treeList.addItem(parentItem, fileItem);
				treeList.focusItem(fileItem);
				// open file
				Tabs.getInstance().openEditor(metaInfo);
			}
			else {
				MMessage.fileError(
					null,
					_("Could not create file"),
					path
				);

				return;
			}
		}
		catch (IOException exception) {
			MMessage.fileError(
				null,
				_("Could not create file"),
				path
			);

			return;
		}
	}

	private void newFolder() {
		TreeListItem parentItem = getSelectedFolderItem();

		if (parentItem == null)
			return;

		MetaInfo metaInfo = parentItem.getData();

		String name = MMessage.input(
			null,
			(lastNewFolderName == null) ? _("New Folder") : lastNewFolderName, // initial value
			_("&Name:"), // label
			_("New Folder"), // title
			"ui/newfolder" // icon
		);

		// cancel?
		if (name == null)
			return;

		lastNewFolderName = name; // history; unescaped

		name = TK.escapeURL(name);

		if (!TreeFS.isFileNameValid(name))
			return;

		String path = FS.makePath(metaInfo.getPath(), name);

		if (pathExists(path))
			return;

		if (!FS.mkdir(path)) {
			MMessage.fileError(
				null,
				_("Could not create folder"),
				path
			);

			return;
		}
		// add item
		TreeListItem folderItem = TreeListItem.createFolderItem(path);
		folderItem.getData().setCreatedTime();
		treeList.addItem(parentItem, folderItem);
		treeList.focusItem(folderItem);
	}

// TODO: 2.0: move to properties dialog
	private void rename() {
		TreeListItem item = treeList.getSelectedItem();

		if (item == null)
			return;

		TreeListItem parentFolder = (TreeListItem)item.getParentItem();

		if (parentFolder == null)
			return;

		MetaInfo metaInfo = item.getData();

		String oldName = metaInfo.toString();
		String newName = MMessage.input(
			null,
			oldName, // initial value
			_("&New Name:"), // label
			_("Rename"), // title
			"rename" // icon
		);

		// cancel?
		if (newName == null)
			return;

		newName = TK.escapeURL(newName);
		oldName = TK.escapeURL(oldName);

		// validate new name
		if (!TreeFS.isFileNameValid(newName) || newName.equals(oldName))
			return;

		if (metaInfo.isFolder())
			newName = FS.makePath(parentFolder.getData().getPath(), newName);
		else
			newName = FS.makePath(parentFolder.getData().getPath(), newName + metaInfo.getType());
		if (!FS.rename(metaInfo.getPath(), newName)) {
			// rename error
			MMessage.fileError(
				null,
				_("Could not rename"),
				newName
			);

			return;
		}

		// file
		if (metaInfo.isFile()) {
			// rename meta info file
			FS.rename(metaInfo.getPath() + ".properties", newName + ".properties");

			// update path
			metaInfo.setPath(newName);
			Tabs.getInstance().setTabInfo(metaInfo);
		}
		// folder
		else {
			// update paths
			final String OLD_PATH = metaInfo.getPath();
			final String NEW_PATH = newName;
			metaInfo.setPath(newName);
			new MTreeScanner<MetaInfo>(item) {
				public void processItem(final MTreeItem<MetaInfo> item) {
					MetaInfo metaInfo = item.getData();
					String path = metaInfo.getPath();
					if (path.startsWith(OLD_PATH + File.separator)) {
						String s = NEW_PATH + path.substring(OLD_PATH.length());
						metaInfo.setPath(s);
					}
				}
			};
		}
		// update gui
		treeList.updateUI();
		infoBar.setTipText(metaInfo.getToolTipText());//!!!common code updateInfo(MetaInfo)
	}

	// private classes

	private final class ColorAction extends AppAction {

		// public

		ColorAction() {
			super(_("&Color..."));
			setHTMLHelp(_("Changes the color of the selected file."));
		}

		public void onAction() {
			color();
		}

	}

	private final class DefaultLabelAction extends AppAction {

		// public

		DefaultLabelAction() {
			super(_("Set &Default Label"));
			setHTMLHelp(_("Sets the default file icon."));
		}

		public void onAction() {
			MetaInfo metaInfo = treeList.getSelectedMetaInfo();

			if (metaInfo == null)
				return;

			metaInfo.setIcon(EditorPlugin.findIconForType(metaInfo.getType()));
			Tabs.getInstance().setTabInfo(metaInfo);
			treeList.updateUI();
		}

	}

	private final class DeleteItemAction extends AppAction {

		// public

		DeleteItemAction() {
			// VK_DELETE may conflict with editor hot keys!
			super(_("&Delete"), "ui/delete", VK_DELETE, CTRL_MASK);
			setHTMLHelp(_("Deletes the selected file or folder.<br><b>NOTE:</b> Locked file cannot be deleted."));
		}

		public void onAction() {
			TreeFS.delete(
				treeList.getSelectedItem(),
				TreeFS.CONFIRM | TreeFS.UPDATE_SELECTION
			);
		}

	}

	private final class LabelAction extends AppAction {

		// public

		LabelAction() {
			super(_("&Label..."));
			setHTMLHelp(_("Changes the label (icon) of the selected file."));
		}

		public void onAction() {
			labelMenu.showPopup(MainWindow.getInstance());//!!!
		}

	}

	private final class LabelItemAction extends AppAction {

		// private

		private String path;

		// public

		LabelItemAction(final String path) {
			this(null, path);
		}

		LabelItemAction(final String text, final String path) {
			super(text, path);
			this.path = path;
		}

		public void onAction() {
			if (path == null)
				return;

			MetaInfo metaInfo = treeList.getSelectedMetaInfo();

			if (metaInfo == null)
				return;

			metaInfo.setIconName(path);
			Tabs.getInstance().setTabInfo(metaInfo);
			treeList.updateUI();
		}

	}

	private final class NewFileAction extends AppAction {

		// public

		NewFileAction() {
			super(_("&New File..."), "newfile", VK_N, CTRL_MASK);
			setHTMLHelp(_("Creates a new file in the selected folder."));
		}

		public void onAction() {
			MainWindow.getInstance().setSidebarVisible(true);
			newFile();
		}

	}

	private final class NewFolderAction extends AppAction {

		// public

		NewFolderAction() {
			super(_("New &Folder..."), "ui/newfolder", VK_N, CTRL_MASK + SHIFT_MASK);
			setHTMLHelp(_("Creates a new subfolder in the selected folder."));
		}

		public void onAction() {
			MainWindow.getInstance().setSidebarVisible(true);
			newFolder();
		}

	}

	private final class PropertiesAction extends AppAction {

		// public

		PropertiesAction() {
			super(_("&Properties..."), null, VK_ENTER, ALT_MASK);
			setHTMLHelp(_("Properties of the selected file or folder."));
		}

		public void onAction() {
			MetaInfo metaInfo = treeList.getSelectedMetaInfo();

			if (metaInfo == null)
				return;

			PropertiesDialog properties = new PropertiesDialog(MainWindow.getInstance(), metaInfo);
			if (properties.isOK()) {
				final Tabs tabs = Tabs.getInstance();
				// update tree
				updateActions();
				treeList.repaint();
				// update tabs
				tabs.setTabInfo(metaInfo);
				tabs.updateState();
				// update editor
				Editor editor = tabs.getTabAt(tabs.findEditor(metaInfo));
				if (editor != null) {
					editor.setReadOnly(metaInfo.isLocked());
					editor.updateActions();
				}
			}
		}

	}

	private final class RenameAction extends AppAction {

		// public

		RenameAction() {
			super(_("&Rename..."), "rename", VK_F2);
			setHTMLHelp(_("Renames the selected file or folder."));
		}

		public void onAction() {
			rename();
		}

	}

	private static final class ToggleBookmarkAction extends AppAction {

		// public

		public enum Mode { ADD, NO_FILE, REMOVE }

		// private

		private Mode mode = Mode.NO_FILE;

		// public

		public ToggleBookmarkAction() {
			super(null, null, OS.isKDE() ? VK_B : VK_D, CTRL_MASK);
		}

		public void onAction() {
			MetaInfo metaInfo = TreeList.getInstance().getSelectedMetaInfo();

			if (metaInfo == null)
				return;

			metaInfo.bookmark.toggle();
			if (metaInfo.bookmark.get()) {
				MStatusBar.setText(_("Added bookmark: {0}", metaInfo.getNicePath()), MStatusBar.AUTOHIDE);
				setMode(Mode.REMOVE, metaInfo);
			}
			else {
				MStatusBar.setWarning(_("Removed bookmark: {0}", metaInfo.getNicePath()));
				setMode(Mode.ADD, metaInfo);
			}
		}

		public void setMode(final Mode mode, final MetaInfo metaInfo) {
			this.mode = mode;
			switch (mode) {
				case ADD:
					setEnabled(true);
					setHTMLHelp(_("Adds the selected file to bookmarks."));
					setIcon("addbookmark");
					setName(_("&Bookmark This File"));
					break;
				case NO_FILE:
					setEnabled(false);
					setHTMLHelp(null);
					setIcon("addbookmark");
					setName(_("&Bookmark This File"));
					break;
				case REMOVE:
					setEnabled(true);
					setHTMLHelp(_("Removes bookmark from the selected file."));
					setIcon("ui/clearright");
					setName(_("Remove &Bookmark"));
					break;
			}
		}

	}

}
